/****************************************************************************************************************************************************
 * Copyright 2022 NXP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *
 *    * Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 *    * Neither the name of the NXP. nor the names of
 *      its contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************************************************************************************/

#include "DevBasicRenderCustomShader.hpp"
#include <FslBase/Log/Log3Fmt.hpp>
#include <FslBase/String/ToVector.hpp>
#include <FslUtil/OpenGLES2/Exceptions.hpp>
#include <FslUtil/OpenGLES2/GLCheck.hpp>
#include <GLES2/gl2.h>

namespace Fsl
{
  namespace
  {
    namespace LocalConfig
    {
      constexpr IO::PathView ShaderBasicVertFixedColorFilename("BasicWithFixedColor.vert");
      constexpr IO::PathView ShaderBasicFragFixedColorFilename("BasicWithFixedColor.frag");
    }

    DevCustomCreateInfo LoadCustomShaders(const std::shared_ptr<IContentManager>& contentManager)
    {
      FSLLOG3_INFO("Loading custom shaders");
      return {contentManager, ToZeroTerminatedVectorUInt8(contentManager->ReadAllText(LocalConfig::ShaderBasicVertFixedColorFilename)),
              ToZeroTerminatedVectorUInt8(contentManager->ReadAllText(LocalConfig::ShaderBasicFragFixedColorFilename))};
    }
  }

  DevBasicRenderCustomShader::DevBasicRenderCustomShader(const DemoAppConfig& config)
    : DemoAppGLES2(config)
    , m_shared(config, LoadCustomShaders(GetContentManager()))
  {
  }


  void DevBasicRenderCustomShader::ConfigurationChanged(const DemoWindowMetrics& windowMetrics)
  {
    m_shared.ConfigurationChanged(windowMetrics);
  }


  void DevBasicRenderCustomShader::Update(const DemoTime& demoTime)
  {
    m_shared.Update(demoTime);
  }


  void DevBasicRenderCustomShader::Draw(const FrameInfo& frameInfo)
  {
    FSL_PARAM_NOT_USED(frameInfo);

    // Clear the screen
    glClearColor(0.5f, 0.5f, 0.5f, 1.0f);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

    // This might affect the currently bound state!
    m_shared.Draw();
  }

}
