#ifndef WINDOW_VULKANTRIANGLE_VULKANTRIANGLE_HPP
#define WINDOW_VULKANTRIANGLE_VULKANTRIANGLE_HPP
/**
 * The MIT License (MIT)
 *
 * Copyright (c) since 2014 Norbert Nopper
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */


// Based on a sample by Norbert Nopper from VKTS Examples (VKTS_Sample02)
// Recreated as a DemoFramework freestyle window sample by Freescale (2016)

#include <FslUtil/Vulkan1_0/VUSwapchainKHR.hpp>
#include <RapidVulkan/Buffer.hpp>
#include <RapidVulkan/CommandBuffer.hpp>
#include <RapidVulkan/CommandPool.hpp>
#include <RapidVulkan/Device.hpp>
#include <RapidVulkan/Framebuffer.hpp>
#include <RapidVulkan/GraphicsPipeline.hpp>
#include <RapidVulkan/ImageView.hpp>
#include <RapidVulkan/Instance.hpp>
#include <RapidVulkan/Memory.hpp>
#include <RapidVulkan/PipelineLayout.hpp>
#include <RapidVulkan/RenderPass.hpp>
#include <RapidVulkan/Semaphore.hpp>
#include <RapidVulkan/ShaderModule.hpp>
#include <Shared/VulkanCustom/VulkanWindowDemoApp.hpp>
#include <vector>

namespace Fsl
{
  class VulkanTriangle : public VulkanWindowDemoApp
  {
    // m_surface
    RapidVulkan::CommandPool m_commandPool;
    RapidVulkan::Buffer m_vertexBuffer;
    RapidVulkan::Memory m_deviceMemoryVertexBuffer;
    RapidVulkan::ShaderModule m_vertexShaderModule;
    RapidVulkan::ShaderModule m_fragmentShaderModule;
    RapidVulkan::PipelineLayout m_pipelineLayout;

    Vulkan::VUSwapchainKHR m_swapchain;
    std::vector<RapidVulkan::ImageView> m_swapchainImageView;
    std::vector<RapidVulkan::Framebuffer> m_framebuffer;
    std::vector<RapidVulkan::CommandBuffer> m_cmdBuffer;
    RapidVulkan::RenderPass m_renderPass;
    RapidVulkan::GraphicsPipeline m_pipeline;

    RapidVulkan::Semaphore m_imageAcquiredSemaphore;
    RapidVulkan::Semaphore m_renderingCompleteSemaphore;

  public:
    explicit VulkanTriangle(const DemoAppConfig& config);
    ~VulkanTriangle() override;

  protected:
    void Update(const DemoTime& demoTime) override;
    void Draw(const FrameInfo& frameInfo) override;

  private:
    void BuildVertexBuffer();
    void BuildShader();
    void BuildPipelineLayout();
    void BuildResources();

    void BuildRenderPass();
    void BuildPipeline();

    void BuildSwapchainImageView(const uint32_t bufferIndex);
    void BuildFramebuffer(const uint32_t bufferIndex);
    void BuildCmdBuffer(const uint32_t bufferIndex);
  };
}

#endif
