/****************************************************************************************************************************************************
 * Copyright 2018, 2022 NXP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *
 *    * Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 *    * Neither the name of the NXP. nor the names of
 *      its contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************************************************************************************/

#include <FslBase/Log/Log3Core.hpp>
#include <FslDemoService/NativeGraphics/Vulkan/NativeGraphicsDeviceShaders.hpp>

namespace Fsl::Vulkan
{
  namespace
  {
    // RenderSystem.frag.spv
    // NOLINTNEXTLINE(modernize-avoid-c-arrays)
    const uint8_t g_shaderFrag[] = {
      3,   2,  35,  7,   0,   0,   1,  0,   10,  0,   8,   0,   24,  0,   0,   0,   0,   0,  0,   0,   17,  0,   2,   0,   1,   0,   0,   0,  11,  0,
      6,   0,  1,   0,   0,   0,   71, 76,  83,  76,  46,  115, 116, 100, 46,  52,  53,  48, 0,   0,   0,   0,   14,  0,   3,   0,   0,   0,  0,   0,
      1,   0,  0,   0,   15,  0,   8,  0,   4,   0,   0,   0,   4,   0,   0,   0,   109, 97, 105, 110, 0,   0,   0,   0,   9,   0,   0,   0,  17,  0,
      0,   0,  21,  0,   0,   0,   16, 0,   3,   0,   4,   0,   0,   0,   7,   0,   0,   0,  3,   0,   3,   0,   2,   0,   0,   0,   194, 1,  0,   0,
      5,   0,  4,   0,   4,   0,   0,  0,   109, 97,  105, 110, 0,   0,   0,   0,   5,   0,  6,   0,   9,   0,   0,   0,   111, 117, 116, 70, 114, 97,
      103, 67, 111, 108, 111, 114, 0,  0,   0,   0,   5,   0,   4,   0,   13,  0,   0,   0,  84,  101, 120, 116, 117, 114, 101, 0,   5,   0,  7,   0,
      17,  0,  0,   0,   105, 110, 70, 114, 97,  103, 84,  101, 120, 116, 117, 114, 101, 67, 111, 111, 114, 100, 0,   0,   5,   0,   5,   0,  21,  0,
      0,   0,  105, 110, 70,  114, 97, 103, 67,  111, 108, 111, 114, 0,   71,  0,   4,   0,  9,   0,   0,   0,   30,  0,   0,   0,   0,   0,  0,   0,
      71,  0,  4,   0,   13,  0,   0,  0,   34,  0,   0,   0,   0,   0,   0,   0,   71,  0,  4,   0,   13,  0,   0,   0,   33,  0,   0,   0,  0,   0,
      0,   0,  71,  0,   4,   0,   17, 0,   0,   0,   30,  0,   0,   0,   1,   0,   0,   0,  71,  0,   4,   0,   21,  0,   0,   0,   30,  0,  0,   0,
      0,   0,  0,   0,   19,  0,   2,  0,   2,   0,   0,   0,   33,  0,   3,   0,   3,   0,  0,   0,   2,   0,   0,   0,   22,  0,   3,   0,  6,   0,
      0,   0,  32,  0,   0,   0,   23, 0,   4,   0,   7,   0,   0,   0,   6,   0,   0,   0,  4,   0,   0,   0,   32,  0,   4,   0,   8,   0,  0,   0,
      3,   0,  0,   0,   7,   0,   0,  0,   59,  0,   4,   0,   8,   0,   0,   0,   9,   0,  0,   0,   3,   0,   0,   0,   25,  0,   9,   0,  10,  0,
      0,   0,  6,   0,   0,   0,   1,  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,  0,   0,   0,   0,   1,   0,   0,   0,   0,   0,  0,   0,
      27,  0,  3,   0,   11,  0,   0,  0,   10,  0,   0,   0,   32,  0,   4,   0,   12,  0,  0,   0,   0,   0,   0,   0,   11,  0,   0,   0,  59,  0,
      4,   0,  12,  0,   0,   0,   13, 0,   0,   0,   0,   0,   0,   0,   23,  0,   4,   0,  15,  0,   0,   0,   6,   0,   0,   0,   2,   0,  0,   0,
      32,  0,  4,   0,   16,  0,   0,  0,   1,   0,   0,   0,   15,  0,   0,   0,   59,  0,  4,   0,   16,  0,   0,   0,   17,  0,   0,   0,  1,   0,
      0,   0,  32,  0,   4,   0,   20, 0,   0,   0,   1,   0,   0,   0,   7,   0,   0,   0,  59,  0,   4,   0,   20,  0,   0,   0,   21,  0,  0,   0,
      1,   0,  0,   0,   54,  0,   5,  0,   2,   0,   0,   0,   4,   0,   0,   0,   0,   0,  0,   0,   3,   0,   0,   0,   248, 0,   2,   0,  5,   0,
      0,   0,  61,  0,   4,   0,   11, 0,   0,   0,   14,  0,   0,   0,   13,  0,   0,   0,  61,  0,   4,   0,   15,  0,   0,   0,   18,  0,  0,   0,
      17,  0,  0,   0,   87,  0,   5,  0,   7,   0,   0,   0,   19,  0,   0,   0,   14,  0,  0,   0,   18,  0,   0,   0,   61,  0,   4,   0,  7,   0,
      0,   0,  22,  0,   0,   0,   21, 0,   0,   0,   133, 0,   5,   0,   7,   0,   0,   0,  23,  0,   0,   0,   19,  0,   0,   0,   22,  0,  0,   0,
      62,  0,  3,   0,   9,   0,   0,  0,   23,  0,   0,   0,   253, 0,   1,   0,   56,  0,  1,   0};
    ;

    // RenderSystem.vert.spv
    // NOLINTNEXTLINE(modernize-avoid-c-arrays)
    const uint8_t g_shaderVert[] = {
      3,   2,   35,  7,   0,   0,   1,   0,   10,  0,   8,   0,  42,  0,   0,   0,   0,   0,   0,   0,   17,  0,   2,   0,   1,   0,   0,   0,   11,
      0,   6,   0,   1,   0,   0,   0,   71,  76,  83,  76,  46, 115, 116, 100, 46,  52,  53,  48,  0,   0,   0,   0,   14,  0,   3,   0,   0,   0,
      0,   0,   1,   0,   0,   0,   15,  0,   11,  0,   0,   0,  0,   0,   4,   0,   0,   0,   109, 97,  105, 110, 0,   0,   0,   0,   10,  0,   0,
      0,   22,  0,   0,   0,   32,  0,   0,   0,   34,  0,   0,  0,   38,  0,   0,   0,   40,  0,   0,   0,   3,   0,   3,   0,   2,   0,   0,   0,
      194, 1,   0,   0,   5,   0,   4,   0,   4,   0,   0,   0,  109, 97,  105, 110, 0,   0,   0,   0,   5,   0,   6,   0,   8,   0,   0,   0,   103,
      108, 95,  80,  101, 114, 86,  101, 114, 116, 101, 120, 0,  0,   0,   0,   6,   0,   6,   0,   8,   0,   0,   0,   0,   0,   0,   0,   103, 108,
      95,  80,  111, 115, 105, 116, 105, 111, 110, 0,   5,   0,  3,   0,   10,  0,   0,   0,   0,   0,   0,   0,   5,   0,   5,   0,   14,  0,   0,
      0,   76,  97,  121, 101, 114, 66,  108, 111, 99,  107, 0,  0,   6,   0,   7,   0,   14,  0,   0,   0,   0,   0,   0,   0,   77,  111, 100, 101,
      108, 86,  105, 101, 119, 80,  114, 111, 106, 0,   0,   0,  5,   0,   6,   0,   16,  0,   0,   0,   103, 95,  112, 117, 115, 104, 67,  111, 110,
      115, 116, 97,  110, 116, 0,   0,   5,   0,   7,   0,   22, 0,   0,   0,   105, 110, 86,  101, 114, 116, 101, 120, 80,  111, 115, 105, 116, 105,
      111, 110, 0,   0,   0,   0,   5,   0,   6,   0,   32,  0,  0,   0,   111, 117, 116, 70,  114, 97,  103, 67,  111, 108, 111, 114, 0,   0,   0,
      0,   5,   0,   6,   0,   34,  0,   0,   0,   105, 110, 86, 101, 114, 116, 101, 120, 67,  111, 108, 111, 114, 0,   0,   0,   5,   0,   7,   0,
      38,  0,   0,   0,   111, 117, 116, 70,  114, 97,  103, 84, 101, 120, 116, 117, 114, 101, 67,  111, 111, 114, 100, 0,   5,   0,   8,   0,   40,
      0,   0,   0,   105, 110, 86,  101, 114, 116, 101, 120, 84, 101, 120, 116, 117, 114, 101, 67,  111, 111, 114, 100, 0,   0,   0,   0,   72,  0,
      5,   0,   8,   0,   0,   0,   0,   0,   0,   0,   11,  0,  0,   0,   0,   0,   0,   0,   71,  0,   3,   0,   8,   0,   0,   0,   2,   0,   0,
      0,   72,  0,   4,   0,   14,  0,   0,   0,   0,   0,   0,  0,   5,   0,   0,   0,   72,  0,   5,   0,   14,  0,   0,   0,   0,   0,   0,   0,
      35,  0,   0,   0,   0,   0,   0,   0,   72,  0,   5,   0,  14,  0,   0,   0,   0,   0,   0,   0,   7,   0,   0,   0,   16,  0,   0,   0,   71,
      0,   3,   0,   14,  0,   0,   0,   2,   0,   0,   0,   71, 0,   4,   0,   22,  0,   0,   0,   30,  0,   0,   0,   0,   0,   0,   0,   71,  0,
      4,   0,   32,  0,   0,   0,   30,  0,   0,   0,   0,   0,  0,   0,   71,  0,   4,   0,   34,  0,   0,   0,   30,  0,   0,   0,   1,   0,   0,
      0,   71,  0,   4,   0,   38,  0,   0,   0,   30,  0,   0,  0,   1,   0,   0,   0,   71,  0,   4,   0,   40,  0,   0,   0,   30,  0,   0,   0,
      2,   0,   0,   0,   19,  0,   2,   0,   2,   0,   0,   0,  33,  0,   3,   0,   3,   0,   0,   0,   2,   0,   0,   0,   22,  0,   3,   0,   6,
      0,   0,   0,   32,  0,   0,   0,   23,  0,   4,   0,   7,  0,   0,   0,   6,   0,   0,   0,   4,   0,   0,   0,   30,  0,   3,   0,   8,   0,
      0,   0,   7,   0,   0,   0,   32,  0,   4,   0,   9,   0,  0,   0,   3,   0,   0,   0,   8,   0,   0,   0,   59,  0,   4,   0,   9,   0,   0,
      0,   10,  0,   0,   0,   3,   0,   0,   0,   21,  0,   4,  0,   11,  0,   0,   0,   32,  0,   0,   0,   1,   0,   0,   0,   43,  0,   4,   0,
      11,  0,   0,   0,   12,  0,   0,   0,   0,   0,   0,   0,  24,  0,   4,   0,   13,  0,   0,   0,   7,   0,   0,   0,   4,   0,   0,   0,   30,
      0,   3,   0,   14,  0,   0,   0,   13,  0,   0,   0,   32, 0,   4,   0,   15,  0,   0,   0,   9,   0,   0,   0,   14,  0,   0,   0,   59,  0,
      4,   0,   15,  0,   0,   0,   16,  0,   0,   0,   9,   0,  0,   0,   32,  0,   4,   0,   17,  0,   0,   0,   9,   0,   0,   0,   13,  0,   0,
      0,   23,  0,   4,   0,   20,  0,   0,   0,   6,   0,   0,  0,   3,   0,   0,   0,   32,  0,   4,   0,   21,  0,   0,   0,   1,   0,   0,   0,
      20,  0,   0,   0,   59,  0,   4,   0,   21,  0,   0,   0,  22,  0,   0,   0,   1,   0,   0,   0,   43,  0,   4,   0,   6,   0,   0,   0,   24,
      0,   0,   0,   0,   0,   128, 63,  32,  0,   4,   0,   30, 0,   0,   0,   3,   0,   0,   0,   7,   0,   0,   0,   59,  0,   4,   0,   30,  0,
      0,   0,   32,  0,   0,   0,   3,   0,   0,   0,   32,  0,  4,   0,   33,  0,   0,   0,   1,   0,   0,   0,   7,   0,   0,   0,   59,  0,   4,
      0,   33,  0,   0,   0,   34,  0,   0,   0,   1,   0,   0,  0,   23,  0,   4,   0,   36,  0,   0,   0,   6,   0,   0,   0,   2,   0,   0,   0,
      32,  0,   4,   0,   37,  0,   0,   0,   3,   0,   0,   0,  36,  0,   0,   0,   59,  0,   4,   0,   37,  0,   0,   0,   38,  0,   0,   0,   3,
      0,   0,   0,   32,  0,   4,   0,   39,  0,   0,   0,   1,  0,   0,   0,   36,  0,   0,   0,   59,  0,   4,   0,   39,  0,   0,   0,   40,  0,
      0,   0,   1,   0,   0,   0,   54,  0,   5,   0,   2,   0,  0,   0,   4,   0,   0,   0,   0,   0,   0,   0,   3,   0,   0,   0,   248, 0,   2,
      0,   5,   0,   0,   0,   65,  0,   5,   0,   17,  0,   0,  0,   18,  0,   0,   0,   16,  0,   0,   0,   12,  0,   0,   0,   61,  0,   4,   0,
      13,  0,   0,   0,   19,  0,   0,   0,   18,  0,   0,   0,  61,  0,   4,   0,   20,  0,   0,   0,   23,  0,   0,   0,   22,  0,   0,   0,   81,
      0,   5,   0,   6,   0,   0,   0,   25,  0,   0,   0,   23, 0,   0,   0,   0,   0,   0,   0,   81,  0,   5,   0,   6,   0,   0,   0,   26,  0,
      0,   0,   23,  0,   0,   0,   1,   0,   0,   0,   81,  0,  5,   0,   6,   0,   0,   0,   27,  0,   0,   0,   23,  0,   0,   0,   2,   0,   0,
      0,   80,  0,   7,   0,   7,   0,   0,   0,   28,  0,   0,  0,   25,  0,   0,   0,   26,  0,   0,   0,   27,  0,   0,   0,   24,  0,   0,   0,
      145, 0,   5,   0,   7,   0,   0,   0,   29,  0,   0,   0,  19,  0,   0,   0,   28,  0,   0,   0,   65,  0,   5,   0,   30,  0,   0,   0,   31,
      0,   0,   0,   10,  0,   0,   0,   12,  0,   0,   0,   62, 0,   3,   0,   31,  0,   0,   0,   29,  0,   0,   0,   61,  0,   4,   0,   7,   0,
      0,   0,   35,  0,   0,   0,   34,  0,   0,   0,   62,  0,  3,   0,   32,  0,   0,   0,   35,  0,   0,   0,   61,  0,   4,   0,   36,  0,   0,
      0,   41,  0,   0,   0,   40,  0,   0,   0,   62,  0,   3,  0,   38,  0,   0,   0,   41,  0,   0,   0,   253, 0,   1,   0,   56,  0,   1,   0};

    // RenderSystemSdf.frag.spv
    // NOLINTNEXTLINE(modernize-avoid-c-arrays)
    const uint8_t g_shaderSdfFrag[] = {
      3,   2,   35,  7,   0,   0,   1,   0,   10,  0,   8,   0,   55,  0,   0,   0,   0,   0,   0,   0,   17,  0,   2,   0,   1,   0,   0,   0,   11,
      0,   6,   0,   1,   0,   0,   0,   71,  76,  83,  76,  46,  115, 116, 100, 46,  52,  53,  48,  0,   0,   0,   0,   14,  0,   3,   0,   0,   0,
      0,   0,   1,   0,   0,   0,   15,  0,   8,   0,   4,   0,   0,   0,   4,   0,   0,   0,   109, 97,  105, 110, 0,   0,   0,   0,   16,  0,   0,
      0,   40,  0,   0,   0,   42,  0,   0,   0,   16,  0,   3,   0,   4,   0,   0,   0,   7,   0,   0,   0,   3,   0,   3,   0,   2,   0,   0,   0,
      194, 1,   0,   0,   5,   0,   4,   0,   4,   0,   0,   0,   109, 97,  105, 110, 0,   0,   0,   0,   5,   0,   5,   0,   8,   0,   0,   0,   100,
      105, 115, 116, 97,  110, 99,  101, 0,   0,   0,   0,   5,   0,   4,   0,   12,  0,   0,   0,   84,  101, 120, 116, 117, 114, 101, 0,   5,   0,
      7,   0,   16,  0,   0,   0,   105, 110, 70,  114, 97,  103, 84,  101, 120, 116, 117, 114, 101, 67,  111, 111, 114, 100, 0,   0,   5,   0,   4,
      0,   23,  0,   0,   0,   97,  108, 112, 104, 97,  0,   0,   0,   5,   0,   5,   0,   25,  0,   0,   0,   76,  97,  121, 101, 114, 66,  108, 111,
      99,  107, 0,   0,   6,   0,   6,   0,   25,  0,   0,   0,   0,   0,   0,   0,   83,  109, 111, 111, 116, 104, 105, 110, 103, 0,   0,   0,   5,
      0,   6,   0,   27,  0,   0,   0,   103, 95,  112, 117, 115, 104, 67,  111, 110, 115, 116, 97,  110, 116, 0,   0,   5,   0,   6,   0,   40,  0,
      0,   0,   111, 117, 116, 70,  114, 97,  103, 67,  111, 108, 111, 114, 0,   0,   0,   0,   5,   0,   5,   0,   42,  0,   0,   0,   105, 110, 70,
      114, 97,  103, 67,  111, 108, 111, 114, 0,   71,  0,   4,   0,   12,  0,   0,   0,   34,  0,   0,   0,   0,   0,   0,   0,   71,  0,   4,   0,
      12,  0,   0,   0,   33,  0,   0,   0,   0,   0,   0,   0,   71,  0,   4,   0,   16,  0,   0,   0,   30,  0,   0,   0,   1,   0,   0,   0,   72,
      0,   5,   0,   25,  0,   0,   0,   0,   0,   0,   0,   35,  0,   0,   0,   64,  0,   0,   0,   71,  0,   3,   0,   25,  0,   0,   0,   2,   0,
      0,   0,   71,  0,   4,   0,   40,  0,   0,   0,   30,  0,   0,   0,   0,   0,   0,   0,   71,  0,   4,   0,   42,  0,   0,   0,   30,  0,   0,
      0,   0,   0,   0,   0,   19,  0,   2,   0,   2,   0,   0,   0,   33,  0,   3,   0,   3,   0,   0,   0,   2,   0,   0,   0,   22,  0,   3,   0,
      6,   0,   0,   0,   32,  0,   0,   0,   32,  0,   4,   0,   7,   0,   0,   0,   7,   0,   0,   0,   6,   0,   0,   0,   25,  0,   9,   0,   9,
      0,   0,   0,   6,   0,   0,   0,   1,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   0,   0,
      0,   0,   27,  0,   3,   0,   10,  0,   0,   0,   9,   0,   0,   0,   32,  0,   4,   0,   11,  0,   0,   0,   0,   0,   0,   0,   10,  0,   0,
      0,   59,  0,   4,   0,   11,  0,   0,   0,   12,  0,   0,   0,   0,   0,   0,   0,   23,  0,   4,   0,   14,  0,   0,   0,   6,   0,   0,   0,
      2,   0,   0,   0,   32,  0,   4,   0,   15,  0,   0,   0,   1,   0,   0,   0,   14,  0,   0,   0,   59,  0,   4,   0,   15,  0,   0,   0,   16,
      0,   0,   0,   1,   0,   0,   0,   23,  0,   4,   0,   18,  0,   0,   0,   6,   0,   0,   0,   4,   0,   0,   0,   21,  0,   4,   0,   20,  0,
      0,   0,   32,  0,   0,   0,   0,   0,   0,   0,   43,  0,   4,   0,   20,  0,   0,   0,   21,  0,   0,   0,   3,   0,   0,   0,   43,  0,   4,
      0,   6,   0,   0,   0,   24,  0,   0,   0,   0,   0,   0,   63,  30,  0,   3,   0,   25,  0,   0,   0,   6,   0,   0,   0,   32,  0,   4,   0,
      26,  0,   0,   0,   9,   0,   0,   0,   25,  0,   0,   0,   59,  0,   4,   0,   26,  0,   0,   0,   27,  0,   0,   0,   9,   0,   0,   0,   21,
      0,   4,   0,   28,  0,   0,   0,   32,  0,   0,   0,   1,   0,   0,   0,   43,  0,   4,   0,   28,  0,   0,   0,   29,  0,   0,   0,   0,   0,
      0,   0,   32,  0,   4,   0,   30,  0,   0,   0,   9,   0,   0,   0,   6,   0,   0,   0,   32,  0,   4,   0,   39,  0,   0,   0,   3,   0,   0,
      0,   18,  0,   0,   0,   59,  0,   4,   0,   39,  0,   0,   0,   40,  0,   0,   0,   3,   0,   0,   0,   32,  0,   4,   0,   41,  0,   0,   0,
      1,   0,   0,   0,   18,  0,   0,   0,   59,  0,   4,   0,   41,  0,   0,   0,   42,  0,   0,   0,   1,   0,   0,   0,   23,  0,   4,   0,   43,
      0,   0,   0,   6,   0,   0,   0,   3,   0,   0,   0,   32,  0,   4,   0,   46,  0,   0,   0,   1,   0,   0,   0,   6,   0,   0,   0,   54,  0,
      5,   0,   2,   0,   0,   0,   4,   0,   0,   0,   0,   0,   0,   0,   3,   0,   0,   0,   248, 0,   2,   0,   5,   0,   0,   0,   59,  0,   4,
      0,   7,   0,   0,   0,   8,   0,   0,   0,   7,   0,   0,   0,   59,  0,   4,   0,   7,   0,   0,   0,   23,  0,   0,   0,   7,   0,   0,   0,
      61,  0,   4,   0,   10,  0,   0,   0,   13,  0,   0,   0,   12,  0,   0,   0,   61,  0,   4,   0,   14,  0,   0,   0,   17,  0,   0,   0,   16,
      0,   0,   0,   87,  0,   5,   0,   18,  0,   0,   0,   19,  0,   0,   0,   13,  0,   0,   0,   17,  0,   0,   0,   81,  0,   5,   0,   6,   0,
      0,   0,   22,  0,   0,   0,   19,  0,   0,   0,   3,   0,   0,   0,   62,  0,   3,   0,   8,   0,   0,   0,   22,  0,   0,   0,   65,  0,   5,
      0,   30,  0,   0,   0,   31,  0,   0,   0,   27,  0,   0,   0,   29,  0,   0,   0,   61,  0,   4,   0,   6,   0,   0,   0,   32,  0,   0,   0,
      31,  0,   0,   0,   131, 0,   5,   0,   6,   0,   0,   0,   33,  0,   0,   0,   24,  0,   0,   0,   32,  0,   0,   0,   65,  0,   5,   0,   30,
      0,   0,   0,   34,  0,   0,   0,   27,  0,   0,   0,   29,  0,   0,   0,   61,  0,   4,   0,   6,   0,   0,   0,   35,  0,   0,   0,   34,  0,
      0,   0,   129, 0,   5,   0,   6,   0,   0,   0,   36,  0,   0,   0,   24,  0,   0,   0,   35,  0,   0,   0,   61,  0,   4,   0,   6,   0,   0,
      0,   37,  0,   0,   0,   8,   0,   0,   0,   12,  0,   8,   0,   6,   0,   0,   0,   38,  0,   0,   0,   1,   0,   0,   0,   49,  0,   0,   0,
      33,  0,   0,   0,   36,  0,   0,   0,   37,  0,   0,   0,   62,  0,   3,   0,   23,  0,   0,   0,   38,  0,   0,   0,   61,  0,   4,   0,   18,
      0,   0,   0,   44,  0,   0,   0,   42,  0,   0,   0,   79,  0,   8,   0,   43,  0,   0,   0,   45,  0,   0,   0,   44,  0,   0,   0,   44,  0,
      0,   0,   0,   0,   0,   0,   1,   0,   0,   0,   2,   0,   0,   0,   65,  0,   5,   0,   46,  0,   0,   0,   47,  0,   0,   0,   42,  0,   0,
      0,   21,  0,   0,   0,   61,  0,   4,   0,   6,   0,   0,   0,   48,  0,   0,   0,   47,  0,   0,   0,   61,  0,   4,   0,   6,   0,   0,   0,
      49,  0,   0,   0,   23,  0,   0,   0,   133, 0,   5,   0,   6,   0,   0,   0,   50,  0,   0,   0,   48,  0,   0,   0,   49,  0,   0,   0,   81,
      0,   5,   0,   6,   0,   0,   0,   51,  0,   0,   0,   45,  0,   0,   0,   0,   0,   0,   0,   81,  0,   5,   0,   6,   0,   0,   0,   52,  0,
      0,   0,   45,  0,   0,   0,   1,   0,   0,   0,   81,  0,   5,   0,   6,   0,   0,   0,   53,  0,   0,   0,   45,  0,   0,   0,   2,   0,   0,
      0,   80,  0,   7,   0,   18,  0,   0,   0,   54,  0,   0,   0,   51,  0,   0,   0,   52,  0,   0,   0,   53,  0,   0,   0,   50,  0,   0,   0,
      62,  0,   3,   0,   40,  0,   0,   0,   54,  0,   0,   0,   253, 0,   1,   0,   56,  0,   1,   0};
  }


  ReadOnlySpan<uint8_t> NativeGraphicsDeviceShaders::GetVertexShader()
  {
    return ReadOnlySpan<uint8_t>(g_shaderVert, sizeof(g_shaderVert));
  }

  ReadOnlySpan<uint8_t> NativeGraphicsDeviceShaders::GetFragmentShader()
  {
    return ReadOnlySpan<uint8_t>(g_shaderFrag, sizeof(g_shaderFrag));
  }

  ReadOnlySpan<uint8_t> NativeGraphicsDeviceShaders::GetSdfFragmentShader()
  {
    return ReadOnlySpan<uint8_t>(g_shaderSdfFrag, sizeof(g_shaderSdfFrag));
  }
}
