#ifndef FSLUTIL_VULKAN1_0_SAFETYPE_BUFFERCREATEINFOCOPY_HPP
#define FSLUTIL_VULKAN1_0_SAFETYPE_BUFFERCREATEINFOCOPY_HPP
/****************************************************************************************************************************************************
 * Copyright (c) 2016 Freescale Semiconductor, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *
 *    * Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 *    * Neither the name of the Freescale Semiconductor, Inc. nor the names of
 *      its contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************************************************************************************/

// Make sure Common.hpp is the first include file (to make the error message as helpful as possible when disabled)
#include <FslUtil/Vulkan1_0/Common.hpp>
#include <vulkan/vulkan.h>
#include <vector>

namespace Fsl::Vulkan
{
  // A 'safe' variant of the VkBufferCreateInfo structure.
  // This one stores the 'pointed' to values inside it
  class BufferCreateInfoCopy
  {
    VkBufferCreateInfo m_value;

    std::vector<uint32_t> m_queueFamilyIndices;

  public:
    BufferCreateInfoCopy(const BufferCreateInfoCopy&) = delete;
    BufferCreateInfoCopy& operator=(const BufferCreateInfoCopy&) = delete;

    BufferCreateInfoCopy& operator=(BufferCreateInfoCopy&& other) noexcept;
    BufferCreateInfoCopy(BufferCreateInfoCopy&& other) noexcept;

    BufferCreateInfoCopy();
    explicit BufferCreateInfoCopy(const VkBufferCreateInfo& value);

    void Reset();
    void Reset(const VkBufferCreateInfo& value);

    const VkBufferCreateInfo& Get() const
    {
      return m_value;
    }

  private:
    // @brief Make sure that the pointers stored in m_value is the correct 'safe' ones
    // @note  This ensures correct behavior when moving etc
    void PatchPointers() noexcept;
  };
}

#endif
