/****************************************************************************************************************************************************
 * Copyright 2022 NXP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *
 *    * Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *
 *    * Neither the name of the NXP. nor the names of
 *      its contributors may be used to endorse or promote products derived from
 *      this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************************************************************************************/

#include "DataGeneratorData.hpp"
#include <FslBase/Span/ReadOnlySpanUtil.hpp>
#include <array>

namespace Fsl
{
  namespace
  {
    using Data = DataGeneratorDataRecord;

    // Name: Vulkan.UI.Benchmark Version: 1.0 Debug: false
    // Scene: 0 Version: 1.0 ResolutionPx: {Width=1920 Height=1080} DensityDpi: 160
    // Entries: UIProcessEvents, UIUpdate, UIDraw, UIDrawPreprocess, UIDrawGenMesh, UIDrawFillBuffers, UIDrawSchedule
    const std::array<Data, 3008> g_entries = {
      Data(0, 102, 11, 7, 23, 10, 2), Data(0, 2, 11, 3, 19, 9, 2),    Data(0, 4, 15, 4, 30, 5, 3),   Data(0, 10, 16, 4, 30, 5, 2),
      Data(0, 4, 14, 4, 27, 5, 2),    Data(0, 4, 13, 4, 25, 5, 2),    Data(0, 4, 14, 4, 24, 5, 2),   Data(0, 4, 13, 4, 24, 5, 2),
      Data(0, 4, 13, 4, 27, 5, 2),    Data(0, 9, 14, 4, 24, 5, 2),    Data(0, 6, 12, 3, 22, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 16, 4, 29, 5, 3),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 15, 4, 26, 5, 2),    Data(0, 0, 16, 4, 25, 5, 2),   Data(0, 0, 13, 4, 25, 5, 2),
      Data(0, 0, 14, 4, 26, 5, 2),    Data(0, 0, 20, 4, 30, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),   Data(0, 0, 13, 4, 25, 5, 2),
      Data(0, 0, 14, 4, 25, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 13, 4, 25, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 12, 3, 22, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 17, 4, 29, 5, 2),   Data(0, 0, 18, 4, 26, 5, 2),
      Data(0, 0, 15, 4, 26, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),    Data(3, 29, 13, 4, 24, 5, 2),  Data(0, 1, 14, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 3),   Data(0, 2, 19, 4, 27, 6, 2),
      Data(0, 1, 14, 4, 24, 5, 2),    Data(0, 10, 14, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 11, 4, 22, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 2, 35, 6, 33, 6, 4),    Data(0, 1, 15, 4, 27, 5, 3),    Data(0, 0, 14, 4, 25, 5, 2),   Data(0, 0, 14, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 12, 4, 23, 5, 2),   Data(0, 0, 13, 3, 22, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 6, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 15, 13, 4, 24, 5, 2),  Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 12, 3, 22, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 26, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 9, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 0, 12, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 12, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 12, 4, 24, 5, 2),
      Data(0, 0, 16, 4, 22, 5, 2),    Data(0, 0, 16, 4, 24, 5, 2),    Data(0, 0, 12, 4, 22, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 26, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 12, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 12, 4, 24, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 1, 12, 4, 24, 5, 2),    Data(0, 16, 13, 4, 24, 5, 2),  Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 14, 4, 24, 5, 2),    Data(0, 1, 12, 4, 24, 5, 2),   Data(0, 1, 12, 4, 24, 5, 2),
      Data(0, 1, 10, 4, 23, 5, 2),    Data(0, 11, 12, 4, 24, 5, 2),   Data(0, 0, 14, 4, 24, 5, 2),   Data(0, 0, 12, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 12, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 11, 4, 22, 5, 2),   Data(0, 0, 12, 4, 24, 5, 2),
      Data(0, 0, 17, 4, 24, 5, 2),    Data(0, 0, 12, 4, 24, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 10, 13, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 1, 10, 3, 18, 5, 2),
      Data(0, 1, 14, 4, 24, 5, 2),    Data(0, 1, 18, 6, 24, 5, 2),    Data(0, 1, 12, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 9, 14, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 14, 4, 26, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 12, 3, 22, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 6, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 6, 2),
      Data(0, 1, 22, 5, 27, 6, 3),    Data(0, 1, 20, 5, 24, 5, 3),    Data(0, 1, 18, 4, 24, 5, 2),   Data(0, 0, 15, 4, 24, 5, 2),
      Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 11, 4, 22, 5, 2),   Data(3, 26, 13, 4, 24, 5, 2),
      Data(0, 1, 14, 4, 26, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 14, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 12, 4, 24, 5, 2),    Data(0, 1, 11, 4, 22, 5, 2),    Data(0, 14, 12, 4, 22, 5, 2),  Data(0, 0, 12, 3, 22, 5, 2),
      Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 14, 4, 24, 7, 4),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 12, 4, 24, 5, 3),   Data(0, 0, 12, 4, 24, 5, 2),
      Data(0, 0, 11, 4, 21, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 19, 5, 27, 7, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),   Data(0, 14, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 12, 4, 24, 5, 2),    Data(0, 1, 12, 4, 24, 5, 2),   Data(0, 1, 18, 4, 24, 5, 5),
      Data(0, 1, 12, 4, 24, 5, 2),    Data(0, 1, 15, 4, 24, 5, 2),    Data(3, 24, 14, 4, 25, 5, 2),  Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 9, 3, 18, 5, 2),    Data(0, 1, 11, 4, 22, 5, 2),
      Data(0, 1, 19, 3, 25, 6, 2),    Data(0, 19, 18, 4, 28, 5, 3),   Data(0, 0, 14, 4, 24, 5, 2),   Data(0, 0, 15, 4, 26, 5, 2),
      Data(0, 0, 13, 4, 26, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 12, 4, 24, 5, 2),   Data(0, 0, 12, 4, 23, 5, 2),
      Data(0, 0, 15, 4, 29, 5, 3),    Data(0, 2, 24, 5, 32, 5, 4),    Data(0, 0, 15, 4, 25, 5, 2),   Data(0, 0, 15, 4, 24, 5, 2),
      Data(0, 1, 21, 5, 24, 6, 3),    Data(0, 1, 19, 4, 25, 5, 3),    Data(0, 1, 16, 4, 25, 5, 3),   Data(0, 0, 15, 4, 24, 5, 2),
      Data(0, 0, 10, 3, 19, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 22, 5, 34, 6, 3),   Data(0, 0, 15, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 21, 15, 4, 26, 5, 2),   Data(0, 1, 14, 4, 25, 5, 2),   Data(0, 1, 14, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 12, 3, 22, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 1, 12, 4, 33, 6, 3),
      Data(0, 13, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 25, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(3, 22, 13, 4, 24, 5, 2),   Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 1, 12, 4, 22, 5, 2),   Data(0, 1, 16, 4, 25, 5, 2),
      Data(0, 1, 16, 5, 32, 5, 3),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 15, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 12, 4, 23, 5, 2),
      Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 26, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 13, 12, 4, 24, 5, 2),  Data(0, 1, 12, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 11, 12, 4, 24, 5, 2),   Data(3, 18, 12, 4, 24, 5, 2),  Data(0, 1, 9, 3, 17, 5, 2),
      Data(0, 1, 14, 4, 26, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 7, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 12, 4, 24, 5, 2),
      Data(0, 0, 12, 4, 24, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 8, 13, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 12, 4, 24, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 1, 14, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 7, 12, 4, 24, 5, 2),    Data(0, 0, 11, 4, 21, 5, 2),   Data(0, 0, 14, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 12, 4, 24, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 12, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 12, 4, 24, 5, 2),   Data(0, 0, 12, 4, 24, 5, 2),
      Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 7, 13, 4, 23, 5, 2),   Data(0, 1, 13, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 1, 12, 4, 24, 5, 2),
      Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 7, 13, 4, 23, 5, 2),    Data(0, 0, 12, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 12, 4, 24, 5, 2),    Data(0, 0, 14, 4, 27, 5, 2),    Data(0, 7, 16, 4, 26, 5, 2),   Data(0, 1, 16, 4, 28, 5, 2),
      Data(0, 1, 14, 4, 25, 5, 2),    Data(0, 1, 14, 4, 24, 5, 2),    Data(0, 1, 14, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 14, 4, 24, 5, 2),    Data(0, 8, 12, 4, 24, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 25, 5, 2),   Data(0, 0, 14, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 7, 13, 4, 24, 5, 2),
      Data(0, 1, 12, 3, 22, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 1, 12, 4, 24, 5, 2),
      Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 7, 12, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(3, 17, 12, 4, 25, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 14, 4, 24, 5, 2),   Data(0, 1, 12, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 7, 12, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 12, 4, 24, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),   Data(0, 7, 13, 4, 24, 5, 2),
      Data(0, 1, 14, 4, 25, 5, 2),    Data(0, 1, 14, 4, 24, 5, 2),    Data(0, 1, 14, 4, 24, 5, 2),   Data(0, 1, 14, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 7, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(2, 16, 13, 4, 24, 5, 2),  Data(0, 1, 11, 3, 18, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 14, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 8, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 7, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 7, 12, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 7, 12, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 12, 4, 24, 5, 2),   Data(0, 7, 13, 4, 23, 5, 2),
      Data(0, 0, 12, 4, 24, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(2, 16, 13, 4, 23, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 1, 14, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 7, 13, 4, 25, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 12, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 14, 4, 25, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 14, 4, 24, 5, 2),
      Data(0, 0, 12, 4, 24, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 7, 12, 4, 24, 5, 2),
      Data(0, 1, 10, 3, 18, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 7, 12, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 8, 13, 4, 24, 5, 2),   Data(0, 1, 14, 4, 24, 5, 2),
      Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 7, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 12, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 12, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 12, 4, 22, 5, 2),    Data(0, 0, 15, 4, 23, 5, 2),   Data(0, 0, 13, 4, 26, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 11, 4, 22, 5, 2),    Data(0, 0, 18, 4, 27, 5, 3),   Data(0, 0, 16, 4, 27, 5, 2),
      Data(0, 0, 14, 4, 25, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 12, 3, 21, 5, 2),   Data(0, 0, 15, 4, 27, 5, 2),
      Data(0, 0, 14, 4, 26, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(4, 24, 12, 4, 24, 5, 2),   Data(0, 2, 13, 4, 24, 5, 2),   Data(0, 1, 12, 3, 21, 5, 2),
      Data(0, 2, 15, 4, 25, 5, 2),    Data(0, 2, 13, 4, 24, 5, 2),    Data(0, 2, 16, 4, 26, 5, 2),   Data(0, 2, 15, 4, 24, 5, 2),
      Data(0, 16, 13, 4, 23, 5, 2),   Data(0, 0, 12, 4, 23, 5, 2),    Data(0, 0, 12, 4, 23, 5, 2),   Data(0, 0, 14, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 12, 13, 4, 23, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 1, 9, 3, 17, 5, 2),
      Data(0, 1, 11, 4, 21, 5, 2),    Data(0, 1, 11, 4, 21, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 10, 12, 4, 24, 5, 2),
      Data(0, 8, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 1, 12, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 8, 13, 4, 27, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 14, 4, 24, 5, 2),
      Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 11, 4, 22, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(3, 18, 12, 4, 24, 5, 2),   Data(0, 2, 13, 4, 24, 5, 2),   Data(0, 2, 13, 4, 24, 5, 2),
      Data(0, 2, 13, 4, 24, 5, 2),    Data(0, 2, 13, 4, 24, 5, 2),    Data(0, 2, 13, 4, 24, 5, 2),   Data(0, 2, 13, 4, 24, 5, 2),
      Data(0, 8, 12, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 14, 4, 26, 5, 2),    Data(0, 0, 14, 4, 26, 5, 2),   Data(0, 0, 12, 4, 22, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 9, 12, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 8, 13, 4, 24, 5, 2),
      Data(0, 7, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 8, 12, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 7, 13, 4, 23, 5, 2),    Data(0, 1, 12, 4, 24, 5, 2),   Data(0, 7, 12, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 9, 3, 17, 5, 2),     Data(0, 0, 14, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 25, 5, 2),    Data(0, 7, 13, 4, 23, 5, 2),   Data(0, 1, 12, 4, 24, 5, 2),
      Data(0, 1, 12, 4, 24, 5, 2),    Data(0, 1, 12, 4, 24, 5, 2),    Data(0, 1, 12, 4, 24, 5, 2),   Data(0, 1, 13, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 7, 12, 4, 24, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 12, 4, 24, 5, 2),    Data(3, 0, 13, 4, 28, 5, 2),    Data(2, 0, 13, 4, 25, 5, 2),   Data(1, 0, 11, 4, 22, 5, 2),
      Data(1, 0, 13, 4, 24, 5, 2),    Data(2, 0, 13, 4, 24, 5, 2),    Data(2, 0, 15, 4, 26, 5, 2),   Data(1, 0, 14, 4, 24, 5, 2),
      Data(1, 0, 14, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(1, 0, 14, 4, 23, 5, 2),
      Data(1, 0, 14, 4, 25, 5, 2),    Data(1, 0, 15, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(1, 0, 12, 4, 24, 5, 2),    Data(1, 0, 13, 4, 24, 5, 2),   Data(1, 0, 13, 4, 24, 5, 2),
      Data(1, 0, 13, 4, 24, 5, 2),    Data(1, 0, 11, 4, 21, 5, 2),    Data(1, 0, 13, 4, 24, 5, 2),   Data(1, 0, 12, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 12, 4, 24, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(2, 0, 12, 4, 24, 5, 2),
      Data(1, 0, 13, 4, 23, 5, 2),    Data(1, 0, 13, 4, 23, 5, 2),    Data(1, 0, 13, 4, 23, 5, 2),   Data(1, 0, 12, 4, 24, 5, 2),
      Data(1, 0, 13, 4, 23, 5, 2),    Data(1, 0, 13, 4, 24, 5, 2),    Data(1, 0, 13, 4, 23, 5, 2),   Data(1, 0, 10, 3, 17, 5, 2),
      Data(1, 0, 14, 4, 26, 5, 2),    Data(1, 0, 15, 4, 27, 5, 2),    Data(1, 0, 14, 4, 25, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(1, 0, 14, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(2, 15, 10, 3, 19, 5, 2),  Data(0, 1, 14, 4, 24, 5, 2),
      Data(0, 1, 14, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 1, 12, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 11, 12, 4, 24, 5, 2),   Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 10, 3, 17, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(2, 0, 13, 4, 24, 5, 2),    Data(1, 0, 14, 4, 24, 5, 2),   Data(1, 0, 14, 4, 23, 5, 2),
      Data(1, 0, 13, 4, 23, 5, 2),    Data(1, 0, 12, 4, 24, 5, 2),    Data(1, 0, 12, 4, 24, 5, 2),   Data(1, 0, 13, 4, 24, 5, 2),
      Data(1, 0, 13, 4, 24, 5, 2),    Data(1, 0, 13, 4, 23, 5, 2),    Data(1, 0, 13, 4, 23, 5, 2),   Data(1, 0, 13, 4, 24, 5, 2),
      Data(1, 0, 13, 4, 24, 5, 2),    Data(1, 0, 12, 4, 24, 5, 2),    Data(1, 0, 13, 4, 23, 5, 2),   Data(1, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 14, 4, 23, 5, 2),
      Data(1, 0, 13, 4, 23, 5, 2),    Data(1, 0, 13, 4, 23, 5, 2),    Data(1, 0, 12, 4, 24, 5, 2),   Data(1, 0, 14, 4, 24, 5, 2),
      Data(1, 0, 13, 4, 24, 5, 2),    Data(1, 0, 12, 4, 21, 5, 2),    Data(1, 0, 13, 4, 23, 5, 2),   Data(0, 0, 11, 3, 22, 5, 2),
      Data(0, 0, 12, 4, 25, 5, 2),    Data(0, 0, 13, 4, 26, 5, 2),    Data(0, 0, 15, 4, 25, 5, 2),   Data(0, 1, 12, 4, 22, 5, 2),
      Data(2, 0, 13, 4, 24, 5, 2),    Data(1, 0, 13, 4, 24, 5, 2),    Data(2, 0, 14, 4, 24, 5, 2),   Data(1, 0, 13, 4, 25, 5, 2),
      Data(2, 0, 11, 4, 22, 5, 2),    Data(0, 0, 12, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 12, 4, 22, 5, 2),
      Data(0, 14, 13, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 12, 3, 21, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 10, 13, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 8, 12, 4, 22, 5, 2),
      Data(0, 1, 19, 4, 25, 5, 2),    Data(0, 1, 17, 4, 24, 5, 2),    Data(0, 1, 14, 4, 24, 5, 2),   Data(0, 1, 12, 4, 22, 5, 2),
      Data(0, 15, 13, 4, 23, 5, 2),   Data(0, 0, 12, 4, 22, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 11, 3, 22, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 12, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 11, 4, 21, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 15, 4, 24, 5, 2),    Data(0, 0, 11, 4, 22, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 11, 4, 22, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),   Data(0, 0, 11, 4, 22, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 17, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 12, 4, 24, 5, 2),    Data(0, 1, 12, 4, 24, 5, 2),   Data(0, 1, 12, 3, 22, 5, 2),
      Data(0, 1, 15, 4, 25, 5, 2),    Data(0, 1, 14, 4, 24, 5, 2),    Data(0, 15, 13, 4, 24, 5, 2),  Data(0, 0, 12, 4, 24, 5, 2),
      Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 14, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 14, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 12, 3, 22, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 12, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 12, 4, 24, 5, 2),    Data(0, 1, 12, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 12, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 9, 13, 4, 23, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 25, 5, 2),    Data(0, 0, 12, 4, 24, 5, 2),   Data(0, 0, 12, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 0, 17, 4, 27, 7, 3),
      Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 14, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 16, 4, 27, 5, 2),
      Data(0, 0, 14, 4, 25, 5, 3),    Data(0, 0, 15, 4, 26, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 10, 13, 4, 24, 5, 2),   Data(0, 1, 12, 4, 22, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 9, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 25, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 12, 3, 21, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 12, 4, 24, 5, 2),
      Data(0, 0, 12, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 12, 4, 24, 5, 2),   Data(0, 0, 13, 4, 26, 5, 2),
      Data(0, 0, 13, 4, 25, 5, 2),    Data(0, 9, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 1, 12, 4, 24, 5, 2),
      Data(0, 1, 12, 4, 24, 5, 2),    Data(0, 1, 11, 4, 22, 5, 2),    Data(0, 1, 12, 4, 22, 5, 2),   Data(0, 1, 16, 4, 28, 5, 2),
      Data(0, 15, 15, 4, 25, 5, 2),   Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(3, 10, 13, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 12, 4, 24, 5, 2),   Data(0, 1, 12, 4, 24, 5, 2),
      Data(0, 1, 12, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 9, 12, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 0, 11, 4, 21, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 14, 4, 27, 5, 2),   Data(0, 0, 15, 4, 25, 5, 2),
      Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 11, 12, 4, 24, 5, 2),  Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 12, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 1, 9, 3, 17, 5, 2),
      Data(0, 1, 12, 4, 24, 5, 2),    Data(0, 8, 11, 4, 21, 5, 2),    Data(0, 0, 13, 4, 26, 5, 2),   Data(0, 0, 12, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 25, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(3, 8, 12, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 1, 15, 4, 26, 5, 2),    Data(0, 1, 12, 4, 23, 5, 2),   Data(0, 1, 13, 4, 26, 5, 2),
      Data(0, 8, 13, 4, 24, 5, 2),    Data(0, 0, 14, 4, 25, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 9, 3, 18, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 12, 4, 24, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 12, 4, 24, 5, 2),
      Data(0, 0, 15, 4, 29, 5, 3),    Data(0, 0, 13, 4, 25, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 9, 13, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 9, 3, 17, 5, 2),     Data(0, 1, 12, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 17, 5, 25, 5, 2),    Data(0, 13, 16, 4, 24, 5, 2),   Data(0, 0, 16, 4, 24, 5, 2),   Data(0, 0, 14, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 12, 4, 24, 5, 2),    Data(0, 1, 34, 6, 31, 7, 4),    Data(0, 2, 30, 6, 26, 6, 4),   Data(0, 1, 26, 5, 27, 6, 3),
      Data(0, 1, 27, 5, 27, 6, 3),    Data(0, 1, 25, 5, 28, 6, 3),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 15, 4, 26, 9, 3),
      Data(0, 0, 17, 4, 26, 6, 2),    Data(0, 1, 19, 4, 28, 5, 3),    Data(0, 1, 17, 4, 26, 5, 2),   Data(0, 1, 16, 4, 25, 5, 2),
      Data(0, 0, 15, 4, 25, 5, 2),    Data(0, 0, 15, 4, 25, 5, 2),    Data(0, 0, 11, 4, 22, 5, 2),   Data(0, 0, 14, 4, 24, 5, 2),
      Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),   Data(0, 0, 15, 4, 24, 5, 2),
      Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 16, 4, 27, 5, 3),    Data(0, 0, 15, 4, 25, 5, 2),   Data(0, 0, 14, 4, 24, 5, 2),
      Data(0, 20, 14, 4, 24, 5, 2),   Data(0, 1, 9, 3, 18, 5, 2),     Data(0, 1, 14, 4, 24, 5, 2),   Data(0, 1, 14, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 14, 4, 24, 5, 2),    Data(0, 1, 15, 4, 25, 5, 2),   Data(0, 18, 14, 4, 24, 5, 2),
      Data(0, 1, 15, 3, 21, 6, 2),    Data(0, 1, 18, 4, 27, 5, 3),    Data(0, 0, 20, 4, 30, 5, 3),   Data(0, 0, 16, 4, 27, 5, 2),
      Data(0, 1, 15, 4, 25, 5, 2),    Data(0, 1, 21, 5, 29, 6, 3),    Data(0, 1, 20, 5, 29, 5, 3),   Data(0, 0, 19, 4, 26, 5, 2),
      Data(0, 0, 17, 4, 24, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),   Data(0, 0, 11, 4, 22, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 21, 13, 4, 24, 5, 2),
      Data(0, 1, 19, 4, 23, 6, 2),    Data(0, 22, 17, 4, 25, 5, 2),   Data(0, 1, 14, 4, 23, 5, 2),   Data(0, 1, 12, 4, 22, 5, 2),
      Data(0, 1, 16, 4, 27, 5, 2),    Data(0, 1, 16, 4, 28, 5, 2),    Data(0, 20, 14, 4, 25, 5, 2),  Data(0, 1, 14, 4, 25, 5, 2),
      Data(0, 15, 14, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 15, 4, 24, 5, 2),   Data(0, 0, 12, 4, 22, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 9, 3, 18, 5, 2),     Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 13, 13, 4, 24, 5, 2),   Data(0, 1, 20, 4, 25, 5, 2),   Data(0, 1, 16, 4, 26, 5, 2),
      Data(0, 1, 12, 3, 19, 5, 2),    Data(0, 1, 13, 4, 22, 5, 2),    Data(0, 1, 14, 4, 25, 5, 2),   Data(0, 1, 14, 4, 24, 5, 2),
      Data(0, 16, 13, 4, 24, 5, 2),   Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 10, 3, 17, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 14, 4, 23, 5, 2),
      Data(0, 0, 17, 4, 22, 5, 2),    Data(0, 0, 18, 4, 24, 5, 2),    Data(0, 0, 16, 4, 23, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 1, 32, 6, 36, 6, 4),    Data(0, 0, 11, 3, 21, 5, 2),    Data(0, 0, 14, 4, 25, 5, 2),   Data(0, 0, 14, 4, 25, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 12, 4, 22, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 12, 4, 22, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 9, 3, 18, 5, 2),     Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 20, 4, 23, 6, 2),    Data(0, 0, 20, 5, 24, 5, 2),    Data(0, 1, 21, 4, 29, 5, 3),   Data(0, 22, 16, 4, 28, 5, 2),
      Data(0, 1, 15, 4, 26, 5, 2),    Data(0, 1, 17, 4, 28, 5, 3),    Data(0, 1, 15, 4, 26, 5, 2),   Data(0, 1, 13, 4, 25, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 19, 15, 4, 24, 5, 2),  Data(82, 29, 12, 4, 22, 6, 2),
      Data(0, 5, 15, 5, 28, 6, 2),    Data(0, 4, 14, 4, 27, 6, 2),    Data(0, 5, 15, 4, 26, 6, 2),   Data(0, 4, 15, 4, 26, 5, 2),
      Data(0, 4, 16, 4, 26, 6, 2),    Data(0, 4, 14, 4, 26, 6, 2),    Data(0, 15, 14, 4, 26, 6, 2),  Data(0, 1, 14, 4, 26, 6, 2),
      Data(0, 11, 14, 4, 26, 6, 2),   Data(0, 0, 16, 4, 26, 6, 2),    Data(0, 0, 15, 4, 26, 6, 2),   Data(0, 0, 15, 4, 26, 6, 2),
      Data(0, 0, 16, 4, 26, 6, 2),    Data(0, 0, 16, 4, 26, 6, 2),    Data(0, 0, 15, 4, 27, 6, 2),   Data(0, 0, 14, 4, 24, 5, 2),
      Data(0, 0, 16, 4, 27, 6, 2),    Data(0, 0, 13, 4, 27, 6, 3),    Data(0, 0, 16, 4, 26, 6, 2),   Data(0, 0, 15, 4, 26, 6, 2),
      Data(0, 0, 18, 4, 30, 6, 3),    Data(0, 0, 17, 4, 28, 5, 3),    Data(0, 0, 13, 5, 30, 6, 2),   Data(0, 0, 15, 4, 27, 6, 2),
      Data(0, 0, 13, 3, 21, 5, 2),    Data(0, 0, 16, 4, 30, 6, 2),    Data(0, 0, 16, 4, 27, 6, 2),   Data(0, 0, 15, 4, 27, 6, 2),
      Data(0, 0, 14, 4, 26, 6, 2),    Data(0, 0, 16, 4, 26, 6, 2),    Data(0, 0, 16, 4, 26, 5, 2),   Data(0, 0, 12, 4, 24, 6, 2),
      Data(0, 0, 16, 4, 26, 5, 2),    Data(0, 0, 15, 4, 26, 6, 2),    Data(0, 0, 14, 4, 26, 6, 2),   Data(0, 0, 17, 4, 30, 10, 3),
      Data(0, 0, 15, 4, 28, 6, 2),    Data(0, 0, 15, 4, 26, 6, 2),    Data(0, 0, 14, 4, 26, 6, 2),   Data(0, 0, 16, 4, 26, 6, 2),
      Data(0, 0, 16, 4, 26, 5, 2),    Data(0, 0, 16, 4, 26, 5, 2),    Data(0, 0, 16, 4, 26, 5, 2),   Data(0, 20, 15, 4, 28, 6, 2),
      Data(0, 1, 15, 4, 26, 6, 2),    Data(0, 1, 15, 4, 26, 6, 2),    Data(0, 1, 14, 4, 26, 6, 2),   Data(0, 1, 14, 4, 26, 6, 2),
      Data(0, 1, 16, 4, 26, 5, 2),    Data(0, 1, 15, 4, 26, 6, 2),    Data(0, 14, 14, 4, 26, 6, 2),  Data(0, 0, 14, 4, 26, 6, 2),
      Data(0, 0, 14, 4, 26, 6, 2),    Data(0, 0, 14, 4, 23, 6, 2),    Data(0, 0, 14, 4, 26, 6, 2),   Data(0, 0, 14, 4, 26, 6, 2),
      Data(0, 0, 14, 4, 26, 6, 2),    Data(0, 0, 14, 4, 26, 6, 2),    Data(0, 0, 15, 4, 26, 6, 2),   Data(0, 0, 15, 4, 26, 6, 2),
      Data(0, 0, 14, 4, 26, 6, 2),    Data(0, 10, 15, 4, 26, 6, 2),   Data(0, 1, 14, 4, 26, 6, 2),   Data(0, 1, 15, 4, 28, 6, 2),
      Data(0, 1, 16, 4, 26, 5, 2),    Data(0, 1, 15, 4, 26, 6, 2),    Data(0, 1, 15, 4, 26, 6, 2),   Data(0, 1, 15, 4, 26, 5, 2),
      Data(3, 12, 15, 4, 26, 6, 2),   Data(0, 3, 15, 4, 26, 5, 2),    Data(0, 2, 13, 4, 26, 6, 2),   Data(0, 2, 14, 4, 27, 5, 2),
      Data(0, 3, 14, 4, 28, 6, 2),    Data(0, 2, 16, 4, 26, 5, 2),    Data(0, 2, 14, 4, 24, 5, 2),   Data(0, 35, 13, 4, 24, 5, 2),
      Data(0, 0, 14, 4, 25, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 14, 4, 23, 5, 2),
      Data(0, 0, 12, 4, 25, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 7, 12, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 12, 4, 24, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 12, 3, 21, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 1, 12, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 12, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 7, 12, 4, 21, 5, 2),    Data(0, 0, 14, 4, 26, 5, 2),    Data(0, 0, 13, 4, 26, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 12, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 12, 4, 24, 5, 2),   Data(0, 0, 12, 4, 24, 5, 2),
      Data(0, 0, 12, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 12, 4, 24, 5, 2),   Data(0, 0, 12, 4, 24, 5, 2),
      Data(0, 7, 12, 4, 24, 5, 2),    Data(0, 1, 12, 4, 24, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 1, 12, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 25, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(82, 29, 14, 4, 24, 6, 2), Data(0, 5, 14, 5, 27, 6, 2),
      Data(0, 5, 15, 4, 26, 6, 2),    Data(0, 4, 15, 4, 26, 6, 2),    Data(0, 4, 15, 4, 26, 6, 2),   Data(0, 4, 14, 4, 26, 6, 2),
      Data(0, 4, 15, 4, 26, 5, 2),    Data(0, 12, 14, 4, 26, 6, 2),   Data(0, 1, 14, 4, 26, 6, 2),   Data(0, 8, 13, 4, 23, 5, 2),
      Data(0, 0, 14, 4, 26, 6, 2),    Data(0, 0, 15, 4, 26, 5, 2),    Data(0, 0, 15, 4, 26, 5, 2),   Data(0, 0, 15, 4, 26, 5, 2),
      Data(0, 0, 14, 4, 26, 6, 2),    Data(0, 0, 15, 4, 26, 5, 2),    Data(0, 0, 14, 4, 26, 6, 2),   Data(0, 0, 14, 4, 26, 6, 2),
      Data(0, 9, 14, 4, 26, 6, 2),    Data(0, 1, 14, 4, 26, 6, 2),    Data(0, 8, 15, 4, 26, 5, 2),   Data(0, 1, 15, 4, 26, 5, 2),
      Data(0, 1, 15, 4, 26, 5, 2),    Data(0, 8, 14, 4, 26, 6, 2),    Data(0, 1, 14, 4, 26, 6, 2),   Data(0, 1, 15, 4, 26, 6, 2),
      Data(0, 8, 14, 4, 26, 6, 2),    Data(0, 1, 15, 4, 26, 6, 2),    Data(0, 1, 14, 4, 26, 6, 2),   Data(0, 1, 15, 4, 26, 5, 2),
      Data(0, 1, 14, 4, 26, 6, 2),    Data(0, 1, 14, 4, 26, 6, 2),    Data(0, 1, 15, 4, 26, 5, 2),   Data(0, 8, 14, 4, 26, 6, 2),
      Data(0, 1, 15, 4, 26, 5, 2),    Data(0, 1, 15, 4, 26, 5, 2),    Data(0, 1, 15, 4, 26, 6, 2),   Data(0, 9, 14, 4, 26, 6, 2),
      Data(0, 1, 14, 4, 28, 6, 2),    Data(0, 1, 15, 4, 28, 5, 2),    Data(0, 8, 15, 4, 26, 5, 2),   Data(0, 0, 15, 4, 26, 5, 2),
      Data(0, 0, 17, 4, 32, 6, 3),    Data(0, 1, 20, 4, 26, 6, 2),    Data(0, 0, 20, 5, 30, 6, 3),   Data(0, 0, 17, 5, 29, 6, 2),
      Data(0, 0, 15, 5, 27, 6, 3),    Data(0, 0, 16, 4, 27, 6, 2),    Data(0, 0, 16, 4, 26, 5, 2),   Data(0, 0, 17, 4, 26, 6, 2),
      Data(0, 0, 15, 4, 28, 6, 2),    Data(0, 0, 15, 4, 26, 6, 2),    Data(0, 0, 15, 4, 26, 6, 2),   Data(0, 0, 15, 4, 26, 6, 2),
      Data(0, 0, 15, 4, 26, 6, 2),    Data(0, 0, 15, 4, 26, 6, 2),    Data(0, 0, 16, 4, 26, 5, 2),   Data(0, 0, 13, 4, 25, 5, 2),
      Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 12, 4, 25, 6, 2),    Data(0, 0, 15, 4, 25, 5, 2),   Data(0, 0, 16, 4, 28, 6, 2),
      Data(0, 0, 16, 4, 26, 6, 2),    Data(0, 0, 14, 4, 25, 6, 2),    Data(0, 0, 16, 4, 25, 5, 2),   Data(0, 0, 17, 4, 29, 7, 2),
      Data(0, 0, 16, 4, 28, 6, 2),    Data(0, 0, 14, 4, 26, 6, 2),    Data(0, 0, 13, 4, 25, 6, 2),   Data(0, 0, 13, 4, 24, 6, 2),
      Data(0, 0, 13, 4, 24, 6, 2),    Data(0, 0, 12, 4, 24, 6, 2),    Data(0, 0, 14, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 12, 4, 24, 6, 2),    Data(0, 0, 13, 4, 24, 6, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 24, 16, 4, 25, 6, 2),   Data(0, 1, 19, 4, 31, 6, 3),    Data(0, 1, 16, 4, 29, 6, 2),   Data(0, 1, 16, 4, 28, 6, 2),
      Data(0, 1, 15, 4, 27, 6, 2),    Data(0, 1, 16, 4, 29, 6, 2),    Data(81, 52, 18, 5, 30, 6, 2), Data(0, 5, 16, 5, 32, 6, 2),
      Data(0, 7, 19, 5, 32, 6, 2),    Data(0, 5, 16, 5, 31, 6, 2),    Data(0, 5, 16, 5, 30, 6, 2),   Data(0, 5, 16, 5, 29, 6, 2),
      Data(0, 4, 11, 4, 21, 6, 2),    Data(0, 19, 15, 5, 29, 6, 2),   Data(0, 9, 16, 5, 29, 6, 2),   Data(0, 0, 15, 5, 29, 6, 2),
      Data(0, 0, 15, 5, 31, 6, 2),    Data(0, 0, 16, 5, 29, 6, 2),    Data(0, 0, 15, 5, 29, 6, 2),   Data(0, 0, 16, 5, 31, 6, 2),
      Data(0, 0, 18, 5, 33, 6, 2),    Data(0, 1, 15, 4, 25, 9, 2),    Data(0, 0, 18, 5, 31, 6, 2),   Data(0, 0, 20, 5, 34, 8, 3),
      Data(0, 0, 17, 5, 30, 6, 2),    Data(0, 0, 16, 5, 30, 6, 2),    Data(0, 0, 16, 5, 30, 6, 2),   Data(0, 0, 17, 5, 29, 6, 2),
      Data(0, 0, 16, 5, 29, 6, 2),    Data(0, 0, 16, 5, 29, 6, 2),    Data(0, 0, 14, 4, 26, 6, 2),   Data(0, 0, 17, 5, 28, 6, 2),
      Data(0, 0, 18, 5, 32, 6, 2),    Data(0, 0, 16, 5, 29, 6, 2),    Data(0, 0, 15, 5, 32, 6, 2),   Data(0, 0, 17, 5, 29, 6, 2),
      Data(0, 1, 18, 5, 29, 6, 2),    Data(0, 0, 16, 5, 29, 6, 2),    Data(0, 0, 16, 5, 28, 6, 2),   Data(0, 16, 17, 5, 32, 6, 2),
      Data(0, 1, 16, 5, 30, 6, 2),    Data(0, 1, 16, 5, 31, 6, 2),    Data(0, 1, 16, 5, 29, 6, 2),   Data(0, 1, 16, 5, 28, 6, 2),
      Data(0, 1, 16, 5, 30, 6, 2),    Data(0, 1, 16, 5, 29, 6, 2),    Data(0, 10, 16, 5, 29, 6, 2),  Data(0, 0, 17, 5, 29, 6, 2),
      Data(0, 0, 16, 5, 28, 6, 2),    Data(0, 9, 18, 5, 28, 6, 2),    Data(0, 1, 19, 5, 34, 6, 3),   Data(0, 1, 18, 5, 30, 6, 2),
      Data(0, 1, 18, 5, 29, 6, 2),    Data(0, 1, 16, 5, 29, 6, 2),    Data(0, 1, 17, 5, 30, 6, 2),   Data(78, 42, 19, 5, 32, 6, 2),
      Data(0, 7, 19, 5, 31, 7, 2),    Data(0, 5, 13, 4, 23, 6, 2),    Data(0, 7, 19, 5, 32, 7, 2),   Data(0, 12, 17, 5, 32, 6, 2),
      Data(0, 7, 16, 5, 31, 7, 2),    Data(0, 7, 17, 5, 31, 6, 2),    Data(0, 12, 16, 5, 31, 7, 2),  Data(0, 10, 17, 5, 31, 7, 2),
      Data(0, 0, 18, 5, 31, 6, 2),    Data(0, 0, 16, 5, 31, 7, 2),    Data(0, 0, 12, 4, 23, 6, 2),   Data(0, 0, 17, 5, 31, 7, 2),
      Data(0, 1, 18, 5, 33, 7, 2),    Data(0, 0, 17, 5, 30, 7, 2),    Data(0, 0, 18, 5, 31, 7, 2),   Data(0, 0, 17, 5, 31, 7, 2),
      Data(0, 1, 17, 5, 30, 7, 2),    Data(0, 1, 17, 5, 32, 7, 2),    Data(0, 0, 17, 5, 31, 7, 2),   Data(0, 0, 15, 5, 28, 7, 2),
      Data(0, 11, 17, 5, 31, 7, 2),   Data(0, 1, 29, 5, 32, 7, 2),    Data(0, 1, 17, 5, 31, 7, 2),   Data(0, 1, 17, 5, 31, 7, 2),
      Data(0, 1, 17, 5, 31, 7, 2),    Data(0, 1, 17, 5, 31, 7, 2),    Data(0, 1, 17, 5, 31, 7, 2),   Data(0, 10, 16, 5, 31, 7, 2),
      Data(0, 1, 17, 5, 31, 7, 2),    Data(0, 10, 16, 5, 32, 7, 2),   Data(0, 10, 17, 5, 31, 7, 2),  Data(0, 1, 18, 5, 31, 6, 2),
      Data(0, 10, 18, 5, 31, 7, 2),   Data(0, 1, 17, 5, 31, 7, 2),    Data(0, 1, 17, 5, 31, 7, 2),   Data(0, 1, 18, 5, 30, 6, 2),
      Data(0, 10, 18, 5, 30, 6, 2),   Data(0, 1, 13, 4, 23, 6, 2),    Data(0, 1, 18, 5, 30, 6, 2),   Data(0, 13, 17, 7, 33, 7, 2),
      Data(0, 0, 17, 5, 30, 7, 2),    Data(0, 1, 17, 5, 30, 7, 2),    Data(0, 0, 18, 5, 30, 6, 2),   Data(0, 0, 18, 5, 30, 7, 2),
      Data(0, 0, 17, 5, 30, 7, 2),    Data(0, 0, 18, 5, 30, 6, 2),    Data(0, 0, 17, 5, 30, 7, 2),   Data(0, 0, 18, 5, 30, 6, 2),
      Data(0, 0, 19, 5, 30, 7, 2),    Data(3, 17, 17, 5, 30, 7, 2),   Data(2, 14, 17, 5, 30, 7, 2),  Data(2, 15, 16, 5, 31, 7, 2),
      Data(2, 14, 18, 5, 30, 6, 2),   Data(2, 14, 17, 5, 31, 7, 2),   Data(2, 14, 17, 5, 31, 7, 2),  Data(1, 10, 12, 4, 23, 6, 2),
      Data(2, 14, 18, 5, 31, 6, 2),   Data(2, 14, 21, 8, 31, 6, 2),   Data(2, 14, 18, 5, 30, 6, 2),  Data(2, 14, 18, 5, 30, 6, 2),
      Data(2, 14, 16, 5, 31, 7, 2),   Data(2, 14, 18, 5, 30, 6, 2),   Data(2, 14, 18, 5, 30, 6, 2),  Data(2, 14, 17, 5, 30, 7, 2),
      Data(2, 14, 21, 5, 30, 6, 2),   Data(2, 14, 18, 5, 30, 6, 2),   Data(2, 14, 20, 5, 32, 6, 4),  Data(2, 14, 18, 5, 30, 7, 2),
      Data(0, 1, 18, 5, 33, 6, 2),    Data(0, 1, 18, 5, 31, 7, 2),    Data(0, 1, 18, 5, 31, 6, 2),   Data(0, 1, 18, 5, 31, 6, 2),
      Data(0, 0, 17, 5, 30, 7, 2),    Data(2, 0, 18, 5, 30, 6, 2),    Data(0, 0, 17, 5, 31, 7, 2),   Data(0, 0, 16, 6, 35, 7, 2),
      Data(0, 0, 16, 5, 31, 7, 2),    Data(0, 0, 18, 5, 30, 6, 2),    Data(0, 0, 17, 5, 33, 7, 2),   Data(0, 10, 18, 5, 34, 7, 2),
      Data(0, 1, 19, 5, 31, 6, 2),    Data(0, 1, 17, 5, 31, 7, 2),    Data(0, 1, 17, 5, 31, 7, 2),   Data(0, 1, 18, 5, 31, 7, 2),
      Data(0, 1, 19, 5, 32, 6, 2),    Data(0, 1, 18, 5, 30, 6, 2),    Data(0, 11, 19, 5, 30, 6, 2),  Data(4, 15, 18, 5, 30, 6, 2),
      Data(2, 14, 18, 5, 30, 6, 2),   Data(2, 14, 17, 5, 31, 7, 2),   Data(2, 14, 17, 5, 31, 7, 2),  Data(2, 14, 17, 5, 30, 7, 2),
      Data(2, 14, 17, 5, 30, 7, 2),   Data(2, 17, 21, 5, 30, 7, 2),   Data(2, 14, 16, 5, 33, 7, 2),  Data(2, 14, 17, 5, 30, 7, 2),
      Data(2, 14, 18, 5, 30, 6, 2),   Data(2, 14, 17, 5, 30, 7, 2),   Data(1, 10, 18, 5, 30, 6, 2),  Data(0, 10, 20, 5, 30, 6, 2),
      Data(0, 1, 17, 5, 30, 7, 2),    Data(0, 1, 17, 5, 30, 7, 2),    Data(0, 1, 18, 5, 33, 6, 2),   Data(0, 1, 18, 5, 30, 6, 2),
      Data(0, 1, 18, 5, 30, 7, 2),    Data(0, 1, 17, 5, 30, 7, 2),    Data(0, 10, 16, 5, 31, 7, 2),  Data(0, 1, 18, 5, 30, 6, 2),
      Data(0, 10, 18, 5, 30, 6, 2),   Data(0, 1, 17, 5, 30, 7, 2),    Data(0, 1, 18, 5, 30, 6, 2),   Data(0, 1, 24, 5, 31, 6, 2),
      Data(0, 1, 17, 5, 31, 6, 2),    Data(0, 10, 19, 5, 30, 6, 2),   Data(0, 1, 18, 5, 30, 7, 2),   Data(0, 0, 17, 5, 30, 7, 2),
      Data(0, 0, 18, 5, 33, 6, 2),    Data(0, 0, 18, 5, 30, 7, 2),    Data(0, 0, 15, 5, 28, 6, 2),   Data(0, 0, 15, 5, 27, 7, 2),
      Data(3, 35, 14, 5, 27, 7, 5),   Data(2, 14, 18, 5, 33, 6, 2),   Data(2, 14, 16, 5, 31, 7, 2),  Data(2, 14, 18, 5, 30, 6, 2),
      Data(2, 14, 18, 5, 30, 6, 2),   Data(2, 14, 18, 5, 30, 6, 2),   Data(2, 14, 17, 5, 30, 6, 2),  Data(2, 14, 18, 5, 30, 6, 2),
      Data(2, 14, 17, 5, 30, 7, 2),   Data(2, 17, 16, 5, 33, 7, 2),   Data(0, 0, 18, 5, 30, 6, 2),   Data(0, 1, 18, 5, 30, 6, 2),
      Data(0, 0, 17, 5, 30, 7, 2),    Data(0, 1, 20, 5, 30, 6, 2),    Data(0, 0, 19, 5, 30, 6, 2),   Data(2, 10, 17, 5, 30, 7, 2),
      Data(0, 1, 18, 5, 31, 6, 2),    Data(0, 1, 18, 5, 30, 6, 2),    Data(0, 1, 20, 5, 31, 8, 4),   Data(0, 10, 19, 5, 30, 6, 2),
      Data(0, 1, 17, 5, 30, 7, 2),    Data(0, 10, 16, 5, 31, 7, 2),   Data(0, 10, 16, 5, 31, 7, 2),  Data(0, 10, 16, 5, 31, 7, 2),
      Data(0, 1, 16, 5, 31, 7, 2),    Data(0, 2, 16, 5, 31, 7, 2),    Data(0, 11, 17, 5, 30, 7, 2),  Data(0, 16, 16, 5, 31, 7, 2),
      Data(0, 11, 18, 5, 31, 7, 2),   Data(0, 11, 17, 5, 31, 7, 2),   Data(0, 1, 18, 5, 31, 7, 2),   Data(0, 10, 18, 5, 31, 6, 2),
      Data(0, 1, 18, 5, 30, 7, 2),    Data(0, 1, 17, 5, 35, 7, 2),    Data(0, 9, 16, 5, 28, 6, 2),   Data(0, 0, 17, 5, 31, 6, 2),
      Data(0, 1, 18, 5, 32, 8, 2),    Data(0, 0, 18, 5, 30, 6, 2),    Data(0, 0, 18, 5, 30, 6, 2),   Data(0, 1, 24, 6, 30, 7, 2),
      Data(0, 1, 24, 5, 30, 6, 2),    Data(0, 1, 19, 4, 29, 6, 2),    Data(0, 1, 18, 6, 34, 7, 2),   Data(0, 0, 16, 4, 24, 6, 2),
      Data(0, 0, 19, 5, 32, 7, 2),    Data(0, 0, 16, 5, 28, 6, 2),    Data(0, 0, 18, 5, 31, 7, 2),   Data(0, 0, 17, 5, 31, 7, 2),
      Data(0, 1, 17, 5, 31, 7, 2),    Data(0, 0, 17, 5, 30, 7, 2),    Data(0, 0, 21, 5, 30, 6, 2),   Data(0, 0, 18, 5, 31, 6, 2),
      Data(0, 0, 18, 5, 30, 6, 2),    Data(0, 0, 18, 5, 30, 7, 2),    Data(0, 23, 18, 5, 30, 6, 2),  Data(0, 1, 18, 5, 30, 7, 2),
      Data(0, 1, 18, 5, 30, 6, 2),    Data(0, 1, 17, 5, 30, 7, 2),    Data(0, 1, 19, 5, 30, 7, 2),   Data(82, 53, 21, 5, 33, 18, 2),
      Data(0, 9, 18, 6, 33, 13, 2),   Data(0, 8, 19, 6, 33, 7, 2),    Data(0, 8, 18, 6, 33, 7, 2),   Data(0, 8, 20, 6, 32, 7, 2),
      Data(0, 8, 18, 6, 33, 7, 2),    Data(0, 8, 18, 6, 33, 7, 2),    Data(0, 16, 18, 6, 33, 7, 2),  Data(0, 12, 19, 5, 33, 7, 2),
      Data(0, 0, 18, 6, 33, 7, 2),    Data(0, 0, 18, 6, 33, 7, 2),    Data(0, 0, 14, 4, 25, 8, 2),   Data(0, 1, 19, 6, 34, 7, 2),
      Data(0, 1, 19, 6, 35, 7, 2),    Data(0, 0, 19, 6, 33, 7, 2),    Data(0, 0, 18, 6, 33, 7, 2),   Data(0, 0, 18, 6, 33, 7, 2),
      Data(0, 0, 18, 6, 33, 7, 2),    Data(0, 0, 18, 6, 33, 7, 2),    Data(0, 13, 17, 6, 33, 7, 2),  Data(0, 1, 18, 6, 33, 7, 2),
      Data(0, 12, 19, 6, 36, 7, 2),   Data(0, 1, 18, 6, 33, 7, 2),    Data(0, 11, 18, 6, 33, 7, 2),  Data(0, 1, 18, 6, 33, 7, 2),
      Data(0, 11, 19, 6, 33, 7, 2),   Data(0, 1, 18, 6, 33, 7, 2),    Data(0, 1, 18, 6, 33, 7, 2),   Data(0, 11, 18, 6, 33, 7, 2),
      Data(0, 1, 18, 6, 33, 7, 2),    Data(0, 11, 18, 6, 33, 7, 2),   Data(0, 1, 20, 6, 33, 7, 2),   Data(0, 11, 18, 6, 33, 7, 2),
      Data(0, 1, 19, 6, 33, 7, 2),    Data(0, 1, 19, 6, 33, 7, 2),    Data(0, 1, 19, 6, 35, 7, 2),   Data(0, 11, 18, 6, 33, 7, 2),
      Data(0, 1, 18, 6, 33, 7, 2),    Data(0, 11, 17, 6, 33, 7, 2),   Data(0, 1, 18, 6, 33, 7, 2),   Data(0, 1, 18, 6, 33, 7, 2),
      Data(0, 1, 18, 6, 33, 7, 2),    Data(0, 1, 18, 6, 33, 7, 2),    Data(0, 12, 18, 5, 31, 7, 2),  Data(0, 13, 18, 5, 31, 7, 2),
      Data(0, 1, 19, 6, 33, 7, 2),    Data(0, 1, 18, 6, 33, 7, 2),    Data(0, 1, 18, 6, 33, 7, 2),   Data(0, 1, 18, 6, 33, 7, 2),
      Data(0, 1, 20, 6, 33, 7, 2),    Data(83, 51, 22, 6, 35, 7, 2),  Data(0, 11, 20, 6, 35, 8, 2),  Data(0, 12, 22, 6, 38, 7, 2),
      Data(0, 10, 21, 6, 36, 7, 2),   Data(0, 10, 20, 6, 35, 8, 2),   Data(0, 10, 19, 6, 35, 8, 2),  Data(0, 10, 20, 6, 35, 7, 2),
      Data(0, 16, 20, 6, 35, 7, 2),   Data(0, 13, 21, 6, 35, 7, 2),   Data(0, 1, 21, 6, 35, 7, 2),   Data(0, 1, 20, 6, 35, 7, 2),
      Data(0, 1, 22, 6, 39, 7, 2),    Data(0, 0, 16, 5, 30, 8, 2),    Data(0, 0, 22, 6, 38, 7, 2),   Data(0, 1, 21, 6, 37, 7, 2),
      Data(0, 0, 21, 6, 36, 7, 2),    Data(0, 18, 21, 6, 35, 7, 2),   Data(0, 1, 21, 6, 35, 7, 2),   Data(0, 1, 21, 6, 35, 7, 2),
      Data(0, 1, 20, 6, 35, 8, 2),    Data(0, 1, 22, 6, 35, 7, 2),    Data(0, 1, 23, 6, 37, 8, 2),   Data(0, 1, 20, 6, 36, 8, 2),
      Data(0, 15, 21, 6, 35, 8, 2),   Data(0, 0, 21, 6, 35, 7, 2),    Data(0, 0, 21, 6, 35, 7, 2),   Data(0, 13, 20, 6, 35, 7, 2),
      Data(0, 1, 20, 6, 35, 7, 2),    Data(0, 13, 20, 6, 35, 7, 2),   Data(0, 1, 21, 6, 35, 7, 2),   Data(0, 1, 20, 6, 35, 7, 2),
      Data(0, 1, 20, 6, 35, 7, 2),    Data(0, 1, 19, 6, 35, 8, 2),    Data(0, 13, 20, 6, 37, 7, 2),  Data(0, 1, 20, 6, 35, 8, 2),
      Data(0, 12, 20, 6, 35, 8, 2),   Data(0, 1, 20, 6, 35, 8, 2),    Data(0, 1, 19, 6, 35, 8, 2),   Data(0, 12, 20, 6, 35, 7, 2),
      Data(0, 1, 19, 6, 35, 8, 2),    Data(0, 1, 21, 6, 37, 7, 2),    Data(0, 1, 20, 6, 35, 8, 2),   Data(0, 1, 21, 6, 35, 7, 2),
      Data(0, 1, 20, 6, 35, 8, 2),    Data(73, 48, 23, 6, 38, 8, 2),  Data(0, 11, 24, 6, 38, 8, 2),  Data(0, 10, 23, 6, 37, 8, 2),
      Data(0, 10, 22, 6, 38, 8, 2),   Data(0, 10, 22, 6, 37, 8, 2),   Data(0, 10, 21, 6, 37, 8, 2),  Data(0, 10, 22, 7, 37, 8, 2),
      Data(0, 15, 20, 7, 37, 8, 2),   Data(0, 1, 21, 7, 37, 8, 2),    Data(0, 13, 21, 6, 37, 8, 2),  Data(0, 0, 22, 6, 37, 8, 2),
      Data(0, 0, 22, 6, 37, 8, 2),    Data(0, 1, 22, 6, 37, 8, 2),    Data(0, 13, 21, 6, 37, 8, 2),  Data(0, 1, 21, 6, 37, 8, 2),
      Data(0, 1, 21, 7, 37, 8, 2),    Data(0, 0, 15, 5, 27, 8, 2),    Data(0, 1, 22, 6, 37, 8, 2),   Data(0, 1, 23, 6, 37, 8, 2),
      Data(0, 1, 22, 6, 37, 8, 2),    Data(0, 13, 22, 6, 37, 8, 2),   Data(0, 0, 22, 6, 37, 8, 2),   Data(0, 0, 22, 6, 37, 8, 2),
      Data(0, 0, 21, 7, 37, 8, 2),    Data(0, 13, 21, 6, 37, 8, 2),   Data(0, 1, 18, 6, 34, 8, 2),   Data(0, 1, 22, 6, 37, 8, 2),
      Data(0, 1, 22, 6, 37, 8, 2),    Data(0, 1, 24, 6, 37, 8, 2),    Data(0, 1, 22, 6, 37, 8, 2),   Data(0, 1, 21, 6, 37, 8, 2),
      Data(0, 21, 22, 6, 37, 8, 2),   Data(0, 1, 24, 6, 34, 8, 2),    Data(0, 1, 24, 7, 38, 8, 2),   Data(0, 1, 23, 7, 37, 8, 2),
      Data(0, 1, 21, 7, 37, 8, 2),    Data(0, 1, 22, 7, 37, 8, 2),    Data(0, 1, 23, 7, 37, 8, 2),   Data(0, 23, 22, 6, 37, 8, 2),
      Data(0, 1, 21, 7, 37, 8, 2),    Data(0, 15, 21, 6, 37, 8, 2),   Data(0, 1, 21, 7, 37, 8, 2),   Data(0, 1, 22, 6, 37, 8, 2),
      Data(0, 1, 21, 7, 37, 8, 2),    Data(0, 1, 21, 7, 37, 8, 2),    Data(0, 1, 21, 6, 37, 8, 2),   Data(0, 1, 23, 6, 37, 8, 2),
      Data(86, 52, 25, 7, 39, 8, 2),  Data(0, 11, 22, 7, 40, 9, 2),   Data(0, 10, 23, 7, 39, 8, 2),  Data(0, 10, 23, 7, 39, 8, 2),
      Data(0, 10, 23, 7, 39, 8, 2),   Data(0, 10, 22, 7, 39, 9, 2),   Data(0, 10, 22, 7, 39, 9, 2),  Data(0, 16, 21, 7, 44, 9, 2),
      Data(0, 1, 20, 7, 38, 9, 2),    Data(0, 15, 23, 7, 40, 9, 2),   Data(0, 1, 22, 7, 40, 9, 2),   Data(0, 0, 22, 7, 39, 9, 2),
      Data(0, 1, 22, 7, 39, 9, 2),    Data(0, 0, 22, 7, 39, 9, 2),    Data(0, 1, 24, 7, 39, 8, 2),   Data(0, 1, 23, 7, 39, 8, 2),
      Data(0, 1, 22, 7, 39, 9, 2),    Data(0, 1, 23, 7, 40, 9, 2),    Data(0, 1, 23, 7, 39, 8, 2),   Data(0, 1, 22, 7, 40, 9, 2),
      Data(0, 1, 21, 7, 40, 9, 2),    Data(0, 1, 21, 7, 40, 9, 2),    Data(0, 0, 25, 7, 43, 8, 2),   Data(0, 0, 16, 5, 30, 8, 2),
      Data(0, 1, 23, 7, 40, 8, 2),    Data(0, 0, 22, 7, 40, 8, 2),    Data(0, 1, 23, 7, 39, 8, 2),   Data(0, 1, 22, 7, 39, 9, 2),
      Data(0, 1, 29, 8, 38, 9, 3),    Data(0, 1, 28, 8, 41, 9, 3),    Data(0, 1, 27, 7, 40, 8, 2),   Data(0, 34, 24, 7, 40, 9, 2),
      Data(0, 1, 19, 5, 29, 8, 2),    Data(0, 1, 22, 7, 40, 9, 2),    Data(0, 1, 23, 7, 39, 9, 2),   Data(0, 1, 26, 7, 45, 9, 2),
      Data(0, 1, 26, 7, 42, 9, 2),    Data(0, 1, 25, 7, 40, 8, 2),    Data(0, 29, 23, 7, 40, 9, 2),  Data(0, 2, 22, 7, 40, 9, 2),
      Data(0, 1, 23, 7, 40, 9, 2),    Data(0, 22, 22, 7, 40, 9, 2),   Data(0, 1, 23, 7, 39, 9, 2),   Data(0, 17, 22, 7, 40, 9, 2),
      Data(0, 1, 22, 7, 40, 9, 2),    Data(0, 1, 23, 7, 40, 8, 2),    Data(0, 1, 24, 6, 41, 9, 2),   Data(0, 1, 24, 8, 46, 9, 2),
      Data(0, 18, 24, 7, 46, 9, 2),   Data(0, 1, 23, 7, 43, 9, 2),    Data(0, 15, 25, 7, 42, 8, 2),  Data(0, 15, 23, 7, 41, 9, 2),
      Data(0, 1, 24, 7, 41, 8, 2),    Data(0, 1, 23, 7, 43, 9, 2),    Data(0, 1, 23, 7, 41, 9, 2),   Data(0, 1, 23, 7, 40, 8, 2),
      Data(0, 1, 23, 7, 42, 9, 2),    Data(90, 67, 26, 7, 43, 20, 2), Data(0, 11, 26, 7, 44, 16, 2), Data(0, 10, 24, 7, 43, 9, 2),
      Data(0, 11, 24, 7, 42, 9, 2),   Data(0, 10, 24, 7, 42, 9, 2),   Data(0, 10, 24, 7, 42, 9, 2),  Data(0, 10, 24, 7, 43, 9, 2),
      Data(0, 17, 24, 7, 42, 9, 2),   Data(0, 1, 24, 7, 42, 9, 2),    Data(0, 15, 24, 7, 42, 9, 2),  Data(0, 1, 24, 7, 42, 9, 2),
      Data(0, 0, 17, 5, 30, 9, 2),    Data(0, 1, 24, 7, 42, 9, 2),    Data(0, 15, 23, 7, 44, 9, 2),  Data(0, 1, 24, 7, 42, 9, 2),
      Data(0, 1, 26, 7, 42, 9, 2),    Data(0, 1, 23, 7, 41, 9, 2),    Data(0, 1, 27, 7, 45, 9, 2),   Data(0, 1, 26, 7, 45, 9, 2),
      Data(0, 1, 25, 7, 43, 9, 2),    Data(0, 19, 24, 7, 44, 9, 2),   Data(0, 1, 25, 7, 42, 9, 2),   Data(0, 1, 27, 7, 43, 9, 2),
      Data(0, 1, 26, 7, 42, 9, 2),    Data(0, 1, 24, 7, 42, 9, 2),    Data(0, 1, 25, 7, 44, 9, 2),   Data(0, 1, 24, 7, 44, 9, 2),
      Data(0, 1, 28, 7, 40, 9, 3),    Data(0, 1, 30, 8, 43, 9, 2),    Data(0, 1, 26, 8, 44, 9, 2),   Data(0, 1, 25, 8, 42, 9, 2),
      Data(0, 1, 27, 7, 42, 9, 2),    Data(0, 1, 25, 7, 43, 9, 2),    Data(0, 1, 24, 7, 42, 9, 2),   Data(0, 1, 25, 7, 42, 9, 2),
      Data(0, 1, 25, 7, 42, 9, 2),    Data(0, 1, 24, 7, 42, 9, 2),    Data(0, 35, 25, 7, 42, 9, 2),  Data(0, 1, 26, 7, 42, 9, 2),
      Data(0, 1, 25, 7, 42, 9, 2),    Data(0, 1, 25, 7, 42, 9, 2),    Data(0, 1, 25, 7, 44, 9, 2),   Data(0, 1, 25, 7, 42, 9, 2),
      Data(133, 66, 26, 8, 46, 9, 3), Data(0, 12, 26, 8, 45, 9, 2),   Data(0, 11, 25, 8, 45, 10, 2), Data(0, 11, 24, 8, 45, 10, 2),
      Data(0, 10, 25, 8, 44, 10, 2),  Data(0, 10, 25, 8, 44, 10, 2),  Data(0, 10, 24, 8, 45, 10, 2), Data(0, 22, 27, 8, 46, 9, 2),
      Data(0, 1, 26, 8, 44, 9, 2),    Data(0, 18, 26, 8, 44, 9, 2),   Data(0, 1, 24, 8, 46, 10, 2),  Data(0, 1, 26, 8, 44, 9, 2),
      Data(0, 1, 26, 8, 44, 9, 2),    Data(0, 1, 24, 8, 44, 10, 2),   Data(0, 1, 27, 8, 44, 9, 2),   Data(0, 1, 25, 8, 45, 9, 2),
      Data(0, 17, 24, 8, 45, 10, 2),  Data(0, 1, 25, 8, 44, 10, 2),   Data(0, 1, 25, 8, 44, 10, 2),  Data(0, 1, 24, 8, 45, 10, 2),
      Data(0, 1, 25, 8, 44, 9, 2),    Data(0, 17, 27, 8, 44, 9, 2),   Data(0, 1, 24, 8, 44, 10, 2),  Data(0, 1, 24, 8, 44, 10, 2),
      Data(0, 17, 24, 8, 44, 10, 2),  Data(0, 1, 24, 8, 46, 10, 2),   Data(0, 1, 25, 8, 47, 9, 2),   Data(0, 17, 26, 8, 45, 9, 2),
      Data(0, 1, 26, 8, 44, 10, 2),   Data(0, 1, 24, 8, 46, 10, 2),   Data(0, 1, 25, 8, 44, 10, 2),  Data(0, 15, 26, 8, 47, 10, 2),
      Data(0, 1, 26, 8, 46, 10, 2),   Data(0, 1, 26, 8, 45, 9, 2),    Data(0, 1, 26, 8, 44, 9, 2),   Data(0, 1, 25, 8, 46, 10, 2),
      Data(4, 23, 24, 8, 45, 10, 2),  Data(0, 1, 24, 8, 45, 10, 2),   Data(0, 1, 25, 8, 44, 10, 2),  Data(2, 21, 23, 8, 47, 10, 2),
      Data(2, 20, 24, 8, 44, 10, 2),  Data(2, 20, 24, 8, 44, 10, 2),  Data(2, 20, 25, 8, 44, 10, 2), Data(2, 20, 24, 8, 45, 10, 2),
      Data(2, 20, 24, 8, 44, 10, 2),  Data(2, 20, 24, 8, 44, 9, 2),   Data(2, 20, 24, 8, 44, 10, 2), Data(2, 20, 24, 8, 44, 10, 2),
      Data(2, 20, 24, 8, 44, 10, 2),  Data(2, 20, 24, 8, 44, 10, 2),  Data(2, 1, 26, 8, 47, 9, 2),   Data(0, 1, 26, 8, 45, 9, 2),
      Data(0, 1, 24, 8, 46, 10, 2),   Data(0, 1, 25, 8, 46, 9, 2),    Data(0, 1, 25, 8, 44, 10, 2),  Data(0, 1, 27, 8, 46, 10, 2),
      Data(0, 1, 24, 8, 45, 10, 2),   Data(0, 16, 26, 8, 46, 9, 2),   Data(0, 1, 26, 8, 45, 10, 2),  Data(0, 1, 25, 8, 45, 9, 2),
      Data(0, 1, 24, 8, 46, 10, 2),   Data(0, 1, 24, 8, 45, 10, 2),   Data(0, 1, 25, 8, 46, 9, 2),   Data(0, 16, 25, 8, 44, 10, 2),
      Data(0, 16, 23, 8, 45, 10, 2),  Data(0, 1, 28, 8, 45, 9, 2),    Data(0, 1, 25, 8, 45, 10, 2),  Data(0, 1, 26, 8, 44, 9, 2),
      Data(0, 1, 25, 8, 44, 9, 2),    Data(0, 1, 26, 8, 44, 9, 2),    Data(0, 16, 24, 8, 46, 10, 2), Data(0, 1, 25, 8, 46, 10, 2),
      Data(2, 1, 24, 8, 45, 10, 2),   Data(2, 21, 25, 8, 45, 9, 2),   Data(2, 20, 24, 8, 45, 10, 2), Data(2, 21, 24, 8, 45, 10, 2),
      Data(2, 20, 24, 8, 45, 9, 2),   Data(2, 20, 24, 8, 44, 10, 2),  Data(2, 20, 26, 8, 46, 9, 2),  Data(2, 20, 25, 8, 45, 9, 2),
      Data(2, 20, 26, 8, 46, 9, 2),   Data(2, 20, 24, 8, 45, 10, 2),  Data(2, 20, 26, 8, 45, 10, 2), Data(2, 20, 24, 8, 44, 10, 2),
      Data(2, 20, 25, 8, 44, 9, 2),   Data(2, 20, 23, 8, 44, 10, 2),  Data(2, 20, 25, 8, 44, 9, 2),  Data(3, 20, 25, 8, 44, 9, 2),
      Data(0, 1, 25, 8, 45, 10, 2),   Data(0, 1, 24, 8, 44, 10, 2),   Data(0, 1, 26, 8, 45, 9, 2),   Data(0, 16, 24, 8, 46, 10, 2),
      Data(0, 1, 26, 8, 45, 9, 2),    Data(0, 1, 26, 8, 45, 9, 2),    Data(0, 16, 26, 8, 47, 10, 2), Data(0, 1, 25, 8, 45, 10, 2),
      Data(0, 16, 25, 8, 44, 10, 2),  Data(0, 1, 25, 8, 46, 10, 2),   Data(0, 16, 23, 8, 45, 10, 2), Data(0, 1, 25, 8, 44, 10, 2),
      Data(0, 1, 25, 8, 45, 9, 2),    Data(0, 16, 24, 8, 45, 10, 2),  Data(0, 16, 25, 8, 44, 9, 2),  Data(0, 12, 18, 6, 32, 9, 2),
      Data(0, 1, 25, 8, 45, 10, 2),   Data(0, 2, 24, 8, 44, 10, 2),   Data(0, 1, 24, 8, 44, 10, 2),  Data(0, 1, 25, 8, 44, 9, 2),
      Data(0, 17, 25, 8, 44, 9, 2),   Data(0, 16, 27, 8, 44, 9, 2),   Data(0, 1, 26, 8, 44, 9, 2),   Data(0, 1, 25, 8, 44, 10, 2),
      Data(0, 1, 26, 8, 44, 9, 2),    Data(0, 1, 25, 8, 44, 10, 2),   Data(0, 1, 25, 8, 44, 10, 2),  Data(0, 1, 26, 8, 48, 10, 2),
      Data(0, 1, 25, 8, 46, 10, 2),   Data(0, 17, 24, 8, 45, 10, 2),  Data(0, 1, 26, 8, 45, 9, 2),   Data(0, 1, 25, 8, 45, 10, 2),
      Data(0, 1, 24, 8, 45, 10, 2),   Data(0, 1, 24, 8, 44, 10, 2),   Data(0, 1, 24, 8, 46, 10, 2),  Data(3, 27, 24, 8, 45, 10, 2),
      Data(0, 9, 24, 8, 45, 10, 2),   Data(0, 9, 24, 8, 45, 10, 2),   Data(0, 9, 25, 8, 44, 9, 2),   Data(0, 9, 24, 8, 44, 10, 2),
      Data(0, 9, 25, 8, 45, 9, 2),    Data(0, 9, 25, 7, 42, 9, 2),    Data(0, 47, 24, 7, 46, 11, 3), Data(0, 1, 24, 7, 42, 9, 2),
      Data(0, 1, 26, 7, 42, 9, 2),    Data(0, 1, 21, 7, 38, 9, 2),    Data(0, 1, 21, 7, 39, 9, 2),   Data(0, 1, 20, 7, 42, 9, 2),
      Data(0, 1, 24, 7, 43, 9, 2),    Data(0, 1, 24, 7, 42, 9, 2),    Data(0, 1, 25, 7, 42, 9, 2),   Data(0, 1, 25, 7, 44, 9, 2),
      Data(0, 1, 26, 7, 43, 9, 2),    Data(0, 1, 24, 7, 42, 9, 2),    Data(0, 1, 24, 7, 42, 9, 2),   Data(0, 1, 24, 7, 42, 9, 2),
      Data(0, 1, 25, 7, 43, 9, 2),    Data(0, 1, 25, 7, 43, 9, 2),    Data(0, 1, 24, 7, 42, 9, 2),   Data(0, 1, 24, 7, 44, 9, 2),
      Data(0, 1, 33, 7, 48, 9, 2),    Data(0, 1, 24, 7, 44, 9, 2),    Data(0, 31, 24, 7, 43, 9, 2),  Data(0, 1, 25, 7, 43, 9, 2),
      Data(0, 1, 24, 7, 42, 9, 2),    Data(0, 20, 24, 7, 45, 9, 2),   Data(0, 1, 25, 7, 43, 9, 2),   Data(0, 1, 25, 7, 42, 9, 2),
      Data(0, 1, 25, 7, 42, 9, 2),    Data(0, 1, 18, 5, 31, 9, 2),    Data(0, 1, 25, 7, 43, 9, 2),   Data(0, 1, 24, 7, 43, 9, 2),
      Data(0, 17, 24, 7, 42, 9, 2),   Data(0, 16, 24, 7, 45, 9, 2),   Data(0, 1, 24, 7, 43, 9, 2),   Data(0, 1, 25, 7, 42, 9, 2),
      Data(0, 1, 24, 7, 42, 9, 2),    Data(0, 1, 24, 7, 42, 9, 2),    Data(0, 1, 26, 7, 45, 9, 2),   Data(0, 1, 25, 7, 43, 9, 2),
      Data(3, 25, 24, 7, 42, 9, 2),   Data(0, 9, 24, 7, 43, 9, 2),    Data(0, 23, 24, 7, 42, 9, 2),  Data(0, 9, 24, 7, 42, 9, 2),
      Data(0, 9, 25, 7, 44, 9, 2),    Data(0, 9, 24, 7, 42, 9, 2),    Data(0, 9, 22, 7, 40, 9, 2),   Data(0, 42, 23, 7, 41, 8, 2),
      Data(0, 14, 22, 7, 40, 8, 2),   Data(0, 1, 21, 7, 40, 9, 2),    Data(0, 1, 23, 7, 39, 8, 2),   Data(0, 1, 23, 7, 39, 8, 2),
      Data(0, 1, 22, 7, 41, 9, 2),    Data(0, 1, 23, 7, 40, 8, 2),    Data(0, 1, 22, 7, 42, 9, 2),   Data(0, 15, 24, 7, 40, 8, 2),
      Data(0, 1, 23, 7, 41, 8, 2),    Data(0, 1, 22, 7, 40, 9, 2),    Data(0, 0, 23, 7, 40, 8, 2),   Data(0, 1, 22, 7, 42, 8, 2),
      Data(0, 1, 23, 7, 42, 8, 2),    Data(0, 1, 22, 7, 41, 8, 2),    Data(0, 14, 22, 7, 41, 8, 2),  Data(0, 1, 22, 7, 40, 9, 2),
      Data(0, 1, 24, 7, 41, 9, 2),    Data(0, 1, 22, 7, 41, 9, 2),    Data(0, 1, 21, 7, 41, 9, 2),   Data(2, 22, 21, 7, 40, 9, 2),
      Data(0, 9, 23, 7, 40, 9, 2),    Data(0, 9, 23, 7, 39, 9, 2),    Data(0, 9, 23, 7, 41, 8, 2),   Data(0, 8, 23, 7, 40, 8, 2),
      Data(0, 8, 22, 7, 40, 8, 2),    Data(0, 8, 22, 6, 37, 8, 2),    Data(0, 43, 22, 6, 37, 8, 2),  Data(0, 0, 22, 6, 38, 8, 2),
      Data(0, 0, 22, 6, 38, 8, 2),    Data(0, 1, 22, 6, 37, 8, 2),    Data(0, 1, 21, 6, 38, 8, 2),   Data(0, 0, 23, 6, 37, 8, 2),
      Data(0, 1, 22, 6, 37, 8, 2),    Data(0, 0, 18, 6, 34, 8, 2),    Data(0, 1, 21, 6, 37, 8, 2),   Data(0, 0, 15, 5, 29, 8, 2),
      Data(0, 1, 21, 6, 39, 8, 2),    Data(0, 1, 21, 6, 38, 8, 2),    Data(0, 1, 22, 6, 37, 8, 2),   Data(0, 1, 21, 6, 40, 8, 2),
      Data(0, 1, 22, 6, 37, 8, 2),    Data(0, 0, 21, 6, 37, 8, 2),    Data(0, 1, 21, 6, 39, 8, 2),   Data(0, 1, 22, 6, 37, 8, 2),
      Data(0, 3, 70, 14, 59, 12, 6),  Data(0, 1, 32, 9, 43, 8, 2),    Data(0, 1, 25, 7, 40, 8, 2),   Data(0, 1, 24, 7, 39, 8, 2),
      Data(0, 1, 22, 7, 38, 8, 2),    Data(0, 1, 22, 7, 39, 8, 2),    Data(0, 0, 22, 7, 38, 8, 2),   Data(0, 1, 22, 6, 38, 8, 2),
      Data(0, 1, 22, 6, 38, 8, 2),    Data(0, 1, 22, 6, 37, 8, 2),    Data(0, 1, 21, 7, 37, 8, 2),   Data(0, 1, 23, 6, 38, 8, 2),
      Data(0, 1, 20, 7, 38, 8, 2),    Data(0, 1, 22, 6, 38, 8, 2),    Data(0, 1, 23, 6, 42, 13, 3),  Data(0, 0, 23, 6, 42, 8, 2),
      Data(0, 1, 22, 6, 39, 8, 2),    Data(0, 1, 24, 6, 39, 8, 2),    Data(0, 1, 22, 6, 38, 8, 2),   Data(0, 1, 23, 6, 37, 8, 2),
      Data(0, 1, 22, 6, 40, 8, 2),    Data(0, 0, 21, 7, 37, 8, 2),    Data(0, 1, 22, 6, 38, 8, 2),   Data(0, 0, 22, 6, 37, 8, 2),
      Data(0, 1, 22, 6, 37, 8, 2),    Data(0, 1, 22, 6, 37, 8, 2),    Data(0, 1, 22, 6, 41, 8, 2),   Data(0, 1, 22, 6, 40, 8, 2),
      Data(0, 1, 22, 6, 39, 8, 2),    Data(0, 1, 22, 6, 38, 8, 2),    Data(0, 1, 22, 6, 38, 8, 2),   Data(0, 1, 22, 6, 37, 8, 2),
      Data(0, 1, 24, 6, 35, 8, 2),    Data(0, 1, 25, 7, 38, 8, 2),    Data(0, 1, 23, 7, 37, 8, 2),   Data(0, 1, 23, 7, 37, 8, 2),
      Data(0, 1, 22, 6, 37, 8, 2),    Data(0, 1, 22, 6, 38, 8, 2),    Data(0, 1, 23, 6, 40, 8, 2),   Data(0, 1, 22, 6, 38, 8, 2),
      Data(0, 38, 22, 6, 37, 8, 2),   Data(0, 1, 22, 6, 37, 8, 2),    Data(0, 20, 22, 6, 37, 8, 2),  Data(0, 15, 21, 7, 40, 8, 2),
      Data(0, 15, 22, 6, 39, 8, 2),   Data(0, 1, 22, 6, 38, 8, 2),    Data(0, 1, 22, 6, 37, 8, 2),   Data(0, 1, 22, 6, 38, 8, 2),
      Data(0, 14, 21, 6, 38, 8, 2),   Data(0, 14, 20, 7, 38, 8, 2),   Data(0, 14, 21, 6, 38, 8, 2),  Data(0, 14, 22, 6, 37, 8, 2),
      Data(0, 1, 22, 6, 38, 8, 2),    Data(0, 1, 22, 6, 37, 8, 2),    Data(0, 1, 21, 6, 37, 8, 2),   Data(0, 13, 22, 6, 37, 8, 2),
      Data(0, 0, 22, 6, 38, 8, 2),    Data(0, 0, 21, 7, 37, 8, 2),    Data(0, 1, 21, 7, 37, 8, 2),   Data(0, 0, 21, 6, 38, 8, 2),
      Data(0, 1, 22, 6, 37, 8, 2),    Data(0, 1, 22, 6, 37, 8, 2),    Data(0, 0, 22, 6, 37, 8, 2),   Data(0, 13, 21, 6, 37, 8, 2),
      Data(0, 13, 21, 6, 38, 8, 2),   Data(0, 1, 22, 6, 37, 8, 2),    Data(0, 1, 21, 6, 38, 8, 3),   Data(0, 1, 23, 6, 38, 8, 2),
      Data(0, 1, 22, 7, 38, 8, 2),    Data(0, 1, 22, 7, 37, 8, 2),    Data(0, 13, 21, 7, 37, 8, 2),  Data(0, 13, 21, 6, 37, 8, 2),
      Data(0, 1, 23, 6, 38, 8, 2),    Data(0, 1, 21, 7, 38, 8, 2),    Data(0, 13, 21, 7, 40, 8, 2),  Data(0, 1, 22, 6, 38, 8, 2),
      Data(0, 1, 21, 6, 38, 8, 2),    Data(0, 1, 21, 6, 38, 8, 2),    Data(0, 1, 22, 6, 37, 8, 2),   Data(0, 1, 22, 6, 37, 8, 2),
      Data(0, 1, 20, 7, 37, 8, 2),    Data(4, 29, 21, 6, 37, 8, 2),   Data(0, 9, 22, 6, 38, 8, 2),   Data(0, 9, 22, 6, 38, 8, 2),
      Data(0, 20, 21, 6, 38, 8, 2),   Data(0, 9, 21, 6, 40, 8, 2),    Data(0, 9, 21, 6, 39, 8, 2),   Data(0, 9, 21, 6, 36, 7, 2),
      Data(0, 43, 19, 6, 36, 8, 2),   Data(0, 0, 20, 6, 36, 7, 2),    Data(0, 0, 20, 6, 35, 8, 2),   Data(0, 1, 21, 6, 35, 7, 2),
      Data(0, 1, 20, 6, 35, 7, 2),    Data(0, 0, 19, 6, 35, 8, 2),    Data(0, 0, 20, 6, 35, 7, 2),   Data(0, 1, 19, 6, 35, 8, 2),
      Data(0, 1, 20, 6, 35, 7, 2),    Data(0, 1, 20, 6, 35, 7, 2),    Data(0, 1, 22, 6, 36, 7, 2),   Data(0, 0, 19, 5, 32, 7, 2),
      Data(0, 1, 20, 6, 38, 7, 2),    Data(0, 1, 20, 6, 35, 7, 2),    Data(0, 1, 21, 6, 35, 7, 2),   Data(0, 0, 20, 6, 35, 7, 2),
      Data(0, 0, 20, 6, 35, 7, 2),    Data(0, 0, 19, 6, 35, 8, 2),    Data(0, 1, 18, 6, 33, 8, 2),   Data(0, 1, 23, 6, 35, 8, 3),
      Data(0, 1, 24, 6, 45, 8, 3),    Data(0, 0, 21, 6, 36, 7, 2),    Data(0, 0, 22, 6, 35, 7, 2),   Data(0, 0, 21, 6, 37, 7, 2),
      Data(0, 1, 20, 6, 35, 8, 2),    Data(0, 1, 20, 6, 35, 8, 2),    Data(0, 1, 21, 6, 35, 7, 2),   Data(0, 1, 19, 6, 35, 8, 2),
      Data(0, 1, 20, 6, 35, 8, 2),    Data(0, 0, 19, 5, 32, 7, 2),    Data(0, 28, 19, 6, 35, 8, 2),  Data(0, 1, 22, 6, 35, 7, 2),
      Data(0, 1, 21, 6, 35, 7, 2),    Data(0, 1, 21, 6, 35, 7, 2),    Data(0, 1, 20, 6, 35, 7, 2),   Data(0, 1, 20, 6, 35, 7, 2),
      Data(0, 1, 20, 6, 39, 8, 2),    Data(0, 1, 19, 6, 36, 8, 2),    Data(0, 1, 20, 6, 35, 8, 2),   Data(0, 1, 19, 6, 37, 8, 2),
      Data(0, 1, 20, 6, 36, 7, 2),    Data(0, 26, 21, 6, 35, 7, 2),   Data(3, 28, 19, 6, 35, 8, 2),  Data(0, 11, 19, 6, 35, 8, 2),
      Data(0, 9, 20, 6, 35, 7, 2),    Data(0, 9, 20, 6, 35, 7, 2),    Data(0, 8, 20, 6, 37, 7, 2),   Data(0, 17, 14, 4, 26, 7, 2),
      Data(0, 9, 19, 6, 34, 7, 2),    Data(0, 45, 18, 6, 33, 7, 2),   Data(0, 1, 18, 6, 33, 7, 2),   Data(0, 1, 19, 6, 33, 7, 2),
      Data(0, 0, 18, 6, 33, 7, 2),    Data(0, 0, 18, 6, 34, 7, 2),    Data(0, 0, 19, 6, 33, 7, 2),   Data(0, 0, 19, 6, 33, 7, 2),
      Data(0, 0, 18, 6, 33, 7, 2),    Data(0, 0, 18, 6, 33, 7, 2),    Data(0, 0, 21, 5, 34, 7, 2),   Data(0, 0, 18, 6, 33, 7, 2),
      Data(0, 1, 18, 6, 33, 7, 2),    Data(0, 1, 18, 6, 33, 7, 2),    Data(0, 0, 19, 6, 32, 7, 2),   Data(0, 0, 19, 6, 32, 7, 2),
      Data(0, 0, 18, 6, 32, 7, 2),    Data(0, 0, 20, 6, 32, 7, 2),    Data(0, 0, 18, 6, 33, 7, 2),   Data(0, 0, 18, 6, 36, 7, 2),
      Data(0, 0, 19, 6, 34, 7, 2),    Data(0, 0, 19, 6, 35, 7, 2),    Data(0, 0, 18, 6, 35, 7, 2),   Data(0, 0, 19, 6, 33, 7, 2),
      Data(0, 0, 18, 6, 33, 7, 2),    Data(0, 1, 19, 6, 32, 7, 2),    Data(0, 0, 22, 6, 38, 7, 3),   Data(0, 0, 19, 5, 33, 7, 2),
      Data(0, 1, 20, 6, 38, 7, 2),    Data(0, 0, 19, 6, 36, 7, 2),    Data(0, 13, 19, 6, 34, 7, 2),  Data(0, 1, 19, 6, 33, 7, 2),
      Data(0, 1, 19, 6, 33, 7, 2),    Data(0, 1, 20, 6, 34, 7, 2),    Data(0, 1, 18, 6, 33, 7, 2),   Data(0, 1, 18, 6, 33, 7, 2),
      Data(0, 1, 18, 6, 34, 7, 2),    Data(0, 1, 18, 6, 33, 7, 2),    Data(0, 13, 19, 5, 33, 7, 2),  Data(0, 0, 18, 6, 35, 7, 2),
      Data(0, 0, 18, 6, 35, 7, 2),    Data(0, 0, 19, 6, 33, 7, 2),    Data(0, 0, 18, 6, 33, 7, 2),   Data(0, 0, 18, 6, 34, 7, 2),
      Data(0, 1, 19, 6, 33, 7, 2),    Data(0, 0, 18, 6, 33, 7, 2),    Data(0, 12, 18, 6, 33, 7, 2),  Data(0, 1, 19, 6, 33, 7, 2),
      Data(0, 1, 18, 6, 35, 7, 2),    Data(0, 1, 18, 6, 33, 7, 2),    Data(0, 1, 19, 5, 33, 7, 2),   Data(0, 1, 18, 6, 33, 7, 2),
      Data(0, 1, 18, 6, 35, 7, 2),    Data(0, 1, 17, 5, 30, 7, 2),    Data(0, 1, 20, 6, 35, 7, 2),   Data(0, 1, 18, 6, 34, 7, 2),
      Data(0, 1, 18, 6, 34, 7, 2),    Data(0, 1, 19, 6, 33, 7, 2),    Data(0, 1, 19, 6, 33, 7, 2),   Data(2, 18, 18, 6, 33, 7, 2),
      Data(0, 7, 18, 6, 33, 7, 2),    Data(0, 7, 18, 6, 33, 7, 2),    Data(0, 6, 18, 6, 35, 7, 2),   Data(0, 7, 19, 5, 33, 7, 2),
      Data(0, 6, 19, 5, 34, 7, 2),    Data(0, 7, 18, 5, 31, 7, 2),    Data(0, 41, 17, 5, 32, 6, 2),  Data(0, 0, 18, 5, 31, 6, 2),
      Data(0, 0, 18, 5, 31, 7, 2),    Data(0, 0, 17, 5, 31, 7, 2),    Data(0, 0, 18, 5, 30, 6, 2),   Data(0, 0, 17, 5, 32, 7, 2),
      Data(0, 0, 17, 5, 31, 7, 2),    Data(0, 0, 17, 5, 31, 7, 2),    Data(0, 1, 17, 5, 31, 7, 2),   Data(0, 0, 17, 5, 30, 7, 2),
      Data(0, 0, 18, 5, 30, 6, 2),    Data(0, 0, 19, 5, 30, 6, 2),    Data(0, 0, 18, 5, 33, 6, 2),   Data(0, 0, 17, 5, 30, 7, 2),
      Data(0, 0, 18, 5, 30, 6, 2),    Data(0, 1, 18, 5, 30, 6, 2),    Data(0, 0, 17, 5, 30, 7, 2),   Data(0, 1, 19, 5, 30, 6, 2),
      Data(0, 1, 19, 5, 30, 6, 2),    Data(0, 1, 20, 5, 30, 6, 2),    Data(0, 0, 18, 5, 30, 6, 2),   Data(0, 0, 17, 5, 30, 7, 2),
      Data(0, 0, 19, 5, 30, 6, 2),    Data(0, 0, 13, 4, 23, 6, 2),    Data(0, 0, 18, 5, 31, 6, 2),   Data(0, 0, 17, 5, 30, 7, 2),
      Data(0, 1, 18, 5, 30, 6, 2),    Data(0, 0, 18, 5, 30, 6, 2),    Data(0, 0, 17, 5, 30, 7, 2),   Data(0, 0, 19, 5, 30, 6, 2),
      Data(0, 1, 18, 5, 30, 6, 2),    Data(0, 11, 18, 5, 30, 6, 2),   Data(0, 1, 18, 5, 31, 6, 2),   Data(0, 1, 16, 5, 31, 7, 2),
      Data(0, 1, 17, 5, 30, 7, 2),    Data(0, 1, 17, 5, 30, 7, 2),    Data(0, 1, 18, 5, 30, 7, 2),   Data(0, 1, 18, 5, 30, 6, 2),
      Data(0, 10, 17, 5, 31, 6, 2),   Data(0, 0, 18, 5, 30, 6, 2),    Data(0, 10, 18, 5, 30, 6, 2),  Data(0, 1, 15, 5, 27, 7, 2),
      Data(0, 1, 20, 6, 31, 7, 2),    Data(0, 1, 19, 5, 30, 7, 2),    Data(0, 1, 20, 5, 35, 7, 2),   Data(0, 1, 14, 4, 35, 6, 3),
      Data(3, 34, 19, 5, 33, 6, 2),   Data(0, 6, 18, 5, 31, 7, 2),    Data(0, 20, 17, 5, 31, 7, 2),  Data(0, 6, 17, 5, 31, 7, 2),
      Data(0, 6, 19, 5, 36, 7, 3),    Data(0, 6, 20, 5, 32, 6, 2),    Data(0, 6, 18, 5, 28, 6, 2),   Data(0, 48, 16, 5, 28, 6, 2),
      Data(0, 0, 16, 5, 28, 6, 2),    Data(0, 0, 16, 5, 28, 6, 2),    Data(0, 0, 15, 5, 28, 6, 2),   Data(0, 0, 16, 5, 28, 6, 2),
      Data(0, 0, 16, 5, 28, 6, 2),    Data(0, 11, 18, 4, 31, 6, 2),   Data(0, 1, 20, 5, 34, 6, 2),   Data(0, 1, 17, 5, 31, 6, 2),
      Data(0, 1, 17, 5, 29, 6, 2),    Data(0, 1, 17, 5, 28, 6, 2),    Data(0, 1, 17, 5, 28, 6, 2),   Data(0, 1, 16, 5, 28, 6, 2),
      Data(0, 10, 16, 5, 28, 6, 2),   Data(0, 0, 16, 5, 28, 6, 2),    Data(0, 0, 16, 5, 28, 6, 2),   Data(0, 0, 15, 5, 28, 6, 2),
      Data(0, 0, 16, 4, 26, 6, 2),    Data(0, 0, 16, 5, 29, 6, 2),    Data(0, 10, 16, 5, 28, 6, 2),  Data(0, 1, 17, 5, 28, 6, 2),
      Data(0, 1, 15, 5, 31, 6, 2),    Data(0, 1, 16, 5, 28, 6, 2),    Data(0, 1, 16, 5, 28, 6, 2),   Data(0, 1, 14, 4, 26, 6, 2),
      Data(3, 15, 15, 5, 28, 6, 2),   Data(0, 4, 15, 5, 28, 6, 2),    Data(0, 4, 17, 5, 31, 6, 2),   Data(0, 4, 16, 5, 30, 6, 2),
      Data(0, 4, 15, 5, 29, 6, 2),    Data(0, 12, 16, 5, 28, 6, 2),   Data(0, 4, 16, 4, 26, 5, 2),   Data(0, 38, 15, 4, 26, 5, 2),
      Data(0, 0, 14, 4, 26, 6, 2),    Data(0, 0, 16, 4, 25, 5, 2),    Data(0, 0, 15, 4, 29, 5, 2),   Data(0, 0, 16, 4, 27, 5, 2),
      Data(0, 0, 17, 4, 26, 5, 2),    Data(0, 0, 15, 4, 26, 5, 2),    Data(0, 0, 15, 4, 26, 5, 2),   Data(0, 0, 16, 4, 26, 5, 2),
      Data(0, 0, 16, 4, 26, 5, 2),    Data(0, 0, 14, 4, 26, 6, 2),    Data(0, 0, 15, 4, 26, 5, 2),   Data(0, 0, 15, 4, 26, 5, 2),
      Data(0, 0, 15, 4, 26, 5, 2),    Data(0, 0, 15, 4, 26, 5, 2),    Data(0, 0, 15, 4, 27, 5, 2),   Data(0, 0, 15, 4, 26, 5, 2),
      Data(0, 0, 15, 5, 36, 5, 2),    Data(0, 0, 15, 4, 26, 5, 2),    Data(0, 0, 15, 4, 26, 5, 2),   Data(0, 0, 15, 4, 26, 5, 2),
      Data(0, 0, 15, 4, 26, 5, 2),    Data(0, 0, 16, 4, 26, 5, 2),    Data(0, 9, 15, 4, 26, 5, 2),   Data(0, 1, 14, 4, 21, 5, 2),
      Data(0, 1, 17, 4, 27, 5, 2),    Data(0, 1, 17, 4, 26, 5, 2),    Data(0, 1, 15, 4, 26, 5, 2),   Data(0, 1, 15, 4, 28, 5, 2),
      Data(0, 1, 15, 4, 26, 5, 2),    Data(0, 9, 14, 4, 26, 6, 2),    Data(0, 0, 15, 4, 26, 5, 2),   Data(0, 0, 14, 4, 26, 6, 2),
      Data(0, 0, 14, 4, 26, 6, 2),    Data(0, 0, 14, 4, 26, 6, 2),    Data(0, 0, 15, 4, 26, 5, 2),   Data(0, 0, 15, 4, 26, 5, 2),
      Data(0, 0, 15, 4, 26, 5, 2),    Data(0, 9, 16, 4, 26, 5, 2),    Data(0, 1, 16, 4, 26, 5, 2),   Data(0, 1, 14, 4, 26, 6, 2),
      Data(0, 1, 14, 4, 26, 6, 2),    Data(0, 1, 15, 4, 26, 6, 2),    Data(3, 11, 15, 4, 26, 5, 2),  Data(0, 3, 15, 4, 26, 5, 2),
      Data(0, 3, 15, 4, 26, 5, 2),    Data(0, 2, 13, 4, 23, 5, 2),    Data(0, 3, 14, 4, 26, 5, 2),   Data(0, 3, 14, 4, 25, 6, 2),
      Data(0, 3, 13, 4, 24, 5, 2),    Data(0, 37, 12, 4, 24, 5, 2),   Data(0, 7, 13, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 1, 13, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 7, 13, 4, 23, 5, 2),   Data(0, 0, 11, 4, 21, 5, 2),
      Data(0, 0, 12, 3, 21, 5, 2),    Data(0, 0, 12, 4, 21, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 0, 14, 4, 23, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 14, 4, 26, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 14, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 14, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 13, 4, 25, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 1, 19, 4, 23, 5, 3),
      Data(0, 1, 18, 4, 24, 5, 2),    Data(0, 1, 16, 4, 23, 5, 2),    Data(0, 0, 15, 4, 23, 5, 2),   Data(0, 0, 13, 4, 21, 5, 2),
      Data(0, 0, 12, 4, 22, 5, 2),    Data(0, 0, 16, 4, 26, 6, 2),    Data(0, 0, 15, 4, 30, 5, 3),   Data(0, 0, 14, 4, 26, 5, 2),
      Data(0, 0, 14, 5, 28, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 14, 4, 23, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 0, 14, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 12, 4, 24, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 1, 22, 5, 27, 5, 3),    Data(0, 1, 21, 4, 26, 5, 3),    Data(0, 20, 14, 4, 19, 5, 2),  Data(0, 1, 19, 4, 29, 5, 3),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 15, 4, 27, 5, 2),    Data(0, 1, 12, 3, 21, 5, 2),   Data(0, 1, 12, 3, 21, 5, 2),
      Data(0, 1, 15, 4, 27, 5, 2),    Data(0, 17, 12, 3, 22, 5, 2),   Data(0, 0, 14, 4, 27, 5, 2),   Data(0, 0, 14, 4, 28, 5, 2),
      Data(0, 0, 13, 3, 22, 5, 2),    Data(0, 0, 13, 3, 22, 5, 2),    Data(0, 0, 13, 4, 25, 5, 2),   Data(3, 27, 14, 4, 27, 5, 2),
      Data(0, 1, 13, 4, 25, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 12, 4, 23, 5, 2),   Data(0, 1, 13, 4, 22, 5, 2),
      Data(0, 1, 12, 4, 22, 5, 2),    Data(0, 1, 12, 4, 22, 5, 2),    Data(0, 18, 15, 4, 27, 7, 2),  Data(0, 0, 15, 4, 23, 5, 2),
      Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 12, 4, 23, 5, 2),    Data(0, 0, 12, 4, 23, 5, 2),   Data(0, 0, 13, 4, 22, 5, 2),
      Data(0, 0, 12, 4, 22, 5, 2),    Data(0, 0, 12, 4, 22, 5, 2),    Data(0, 0, 12, 4, 22, 5, 2),   Data(0, 0, 12, 4, 22, 5, 2),
      Data(0, 0, 12, 4, 22, 5, 2),    Data(0, 0, 12, 4, 22, 5, 2),    Data(0, 16, 11, 4, 23, 5, 2),  Data(0, 1, 12, 4, 22, 5, 2),
      Data(0, 1, 13, 4, 22, 5, 2),    Data(0, 1, 12, 4, 22, 5, 2),    Data(0, 1, 12, 4, 22, 5, 2),   Data(0, 1, 12, 4, 22, 5, 2),
      Data(0, 1, 13, 4, 22, 5, 2),    Data(0, 12, 12, 4, 22, 5, 2),   Data(0, 0, 12, 4, 25, 5, 2),   Data(3, 19, 11, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 22, 5, 2),    Data(0, 1, 11, 3, 20, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 1, 12, 4, 23, 5, 2),
      Data(0, 1, 12, 4, 23, 5, 2),    Data(0, 1, 12, 4, 22, 5, 2),    Data(0, 9, 12, 4, 22, 5, 2),   Data(0, 0, 12, 4, 22, 5, 2),
      Data(0, 0, 12, 4, 22, 5, 2),    Data(0, 7, 13, 4, 22, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 1, 12, 4, 23, 5, 2),
      Data(0, 1, 17, 4, 23, 5, 2),    Data(0, 1, 16, 4, 22, 5, 2),    Data(0, 15, 14, 4, 23, 5, 2),  Data(0, 1, 14, 4, 22, 5, 2),
      Data(0, 11, 12, 4, 22, 5, 2),   Data(0, 1, 13, 4, 22, 5, 2),    Data(0, 1, 12, 4, 22, 5, 2),   Data(0, 1, 12, 4, 22, 5, 2),
      Data(0, 13, 13, 4, 22, 5, 2),   Data(0, 1, 12, 4, 22, 5, 2),    Data(0, 1, 12, 4, 22, 5, 2),   Data(0, 1, 12, 3, 21, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 12, 4, 23, 5, 2),    Data(0, 1, 12, 4, 22, 5, 2),   Data(0, 9, 16, 4, 27, 5, 2),
      Data(0, 0, 11, 3, 21, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 11, 4, 23, 5, 2),   Data(0, 0, 12, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 12, 4, 22, 5, 2),    Data(0, 0, 12, 4, 22, 5, 2),   Data(0, 0, 12, 4, 22, 5, 2),
      Data(0, 0, 12, 4, 22, 5, 2),    Data(0, 0, 13, 4, 22, 5, 2),    Data(0, 0, 12, 4, 22, 5, 2),   Data(0, 7, 13, 4, 22, 5, 2),
      Data(0, 1, 12, 4, 22, 5, 2),    Data(0, 1, 15, 4, 27, 5, 2),    Data(0, 7, 12, 4, 22, 5, 2),   Data(0, 1, 12, 4, 22, 5, 2),
      Data(0, 1, 12, 4, 22, 5, 2),    Data(0, 1, 13, 4, 22, 5, 2),    Data(0, 7, 12, 4, 22, 5, 2),   Data(0, 1, 12, 4, 22, 5, 2),
      Data(0, 1, 12, 4, 22, 5, 2),    Data(0, 7, 12, 4, 22, 5, 2),    Data(0, 0, 11, 4, 23, 5, 2),   Data(0, 0, 12, 4, 22, 5, 2),
      Data(0, 7, 12, 4, 22, 5, 2),    Data(0, 1, 12, 4, 22, 5, 2),    Data(0, 1, 12, 4, 22, 5, 2),   Data(0, 1, 12, 4, 22, 5, 2),
      Data(0, 1, 13, 4, 22, 5, 2),    Data(0, 1, 12, 4, 22, 5, 2),    Data(0, 1, 12, 4, 22, 5, 2),   Data(0, 10, 12, 4, 22, 5, 2),
      Data(0, 1, 12, 4, 22, 5, 2),    Data(0, 1, 11, 4, 22, 5, 2),    Data(0, 1, 12, 4, 22, 5, 2),   Data(0, 1, 12, 4, 22, 5, 2),
      Data(0, 1, 12, 4, 22, 5, 2),    Data(0, 1, 12, 4, 22, 5, 2),    Data(0, 10, 13, 3, 23, 5, 2),  Data(0, 0, 15, 4, 27, 5, 2),
      Data(0, 0, 16, 4, 24, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 0, 14, 4, 23, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 15, 4, 26, 5, 2),    Data(0, 1, 14, 4, 25, 5, 2),   Data(0, 0, 14, 4, 24, 5, 2),
      Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 10, 16, 4, 27, 5, 3),   Data(0, 1, 14, 4, 25, 5, 2),   Data(0, 1, 14, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 14, 4, 24, 5, 2),    Data(0, 9, 13, 4, 24, 5, 2),   Data(0, 1, 13, 4, 26, 5, 2),
      Data(0, 8, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 25, 5, 2),   Data(0, 1, 14, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 8, 14, 4, 24, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 27, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 15, 4, 24, 6, 5),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 25, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 3, 21, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 9, 13, 4, 23, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 1, 14, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 8, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 8, 13, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 1, 13, 4, 23, 5, 2),
      Data(0, 8, 16, 4, 27, 5, 2),    Data(0, 0, 14, 4, 25, 5, 2),    Data(0, 0, 14, 4, 27, 5, 2),   Data(0, 0, 15, 4, 25, 5, 2),
      Data(0, 0, 12, 4, 22, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 15, 4, 29, 8, 3),   Data(3, 23, 13, 4, 23, 5, 2),
      Data(0, 1, 14, 4, 25, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 14, 4, 25, 5, 2),   Data(0, 1, 14, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 12, 13, 4, 23, 5, 2),  Data(0, 8, 14, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 7, 14, 4, 23, 5, 2),
      Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 8, 14, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 8, 14, 4, 23, 5, 2),   Data(0, 0, 14, 4, 23, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(3, 17, 12, 4, 24, 5, 2),   Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),   Data(0, 1, 14, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 7, 13, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 15, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 14, 4, 23, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 7, 13, 4, 23, 5, 2),
      Data(0, 7, 14, 4, 23, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),   Data(0, 1, 14, 4, 23, 5, 2),
      Data(0, 8, 14, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 11, 13, 4, 23, 5, 2),  Data(0, 12, 13, 4, 23, 5, 2),
      Data(0, 12, 14, 4, 23, 5, 2),   Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 7, 13, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 8, 13, 4, 23, 5, 2),    Data(0, 7, 13, 4, 23, 5, 2),   Data(0, 7, 13, 4, 23, 5, 2),
      Data(0, 11, 13, 4, 24, 5, 2),   Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 1, 13, 4, 23, 5, 2),
      Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 7, 13, 4, 23, 5, 2),   Data(0, 7, 13, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 7, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 1, 12, 3, 21, 5, 2),   Data(0, 8, 14, 4, 23, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(3, 0, 13, 4, 23, 5, 2),    Data(2, 0, 13, 4, 23, 5, 2),   Data(1, 0, 14, 4, 23, 5, 2),
      Data(1, 0, 13, 4, 23, 5, 2),    Data(1, 0, 13, 4, 23, 5, 2),    Data(1, 0, 13, 4, 23, 5, 2),   Data(1, 0, 12, 4, 21, 5, 2),
      Data(1, 0, 17, 4, 23, 5, 2),    Data(0, 0, 16, 4, 29, 5, 3),    Data(0, 0, 13, 4, 25, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(2, 14, 13, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 1, 13, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 11, 13, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 8, 13, 4, 23, 5, 2),   Data(0, 1, 14, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 7, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 2, 13, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 1, 14, 4, 23, 5, 2),
      Data(0, 7, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 12, 3, 21, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(2, 0, 13, 4, 23, 5, 2),    Data(2, 0, 13, 4, 23, 5, 2),    Data(1, 0, 13, 4, 23, 5, 2),   Data(1, 0, 13, 4, 23, 5, 2),
      Data(1, 0, 13, 4, 23, 5, 2),    Data(1, 0, 13, 4, 23, 5, 2),    Data(1, 0, 12, 4, 24, 5, 2),   Data(1, 0, 13, 4, 23, 5, 2),
      Data(1, 0, 13, 4, 23, 5, 2),    Data(1, 0, 13, 4, 26, 5, 2),    Data(1, 0, 14, 4, 23, 5, 2),   Data(1, 0, 13, 4, 23, 5, 2),
      Data(1, 0, 14, 4, 23, 5, 2),    Data(1, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 16, 4, 28, 8, 3),   Data(0, 0, 13, 4, 26, 5, 2),
      Data(0, 0, 15, 4, 27, 5, 2),    Data(0, 0, 14, 4, 26, 5, 2),    Data(0, 0, 13, 4, 26, 5, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 12, 3, 22, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 8, 13, 4, 23, 5, 2),   Data(0, 1, 13, 4, 23, 5, 2),
      Data(0, 1, 21, 4, 22, 5, 2),    Data(0, 1, 20, 4, 24, 5, 2),    Data(0, 17, 18, 4, 23, 5, 2),  Data(0, 1, 17, 4, 23, 5, 2),
      Data(0, 1, 16, 4, 24, 5, 2),    Data(0, 14, 14, 4, 24, 5, 2),   Data(0, 1, 15, 4, 23, 5, 2),   Data(0, 1, 15, 4, 23, 5, 2),
      Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 11, 13, 4, 24, 5, 2),   Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 9, 14, 4, 23, 5, 2),
      Data(0, 1, 14, 4, 23, 5, 2),    Data(4, 19, 13, 4, 23, 5, 2),   Data(3, 13, 13, 4, 23, 5, 2),  Data(2, 12, 13, 4, 23, 5, 2),
      Data(2, 12, 13, 4, 23, 5, 2),   Data(2, 12, 12, 4, 24, 5, 2),   Data(2, 16, 12, 4, 24, 5, 2),  Data(2, 11, 12, 4, 23, 5, 2),
      Data(2, 8, 8, 3, 17, 5, 2),     Data(2, 11, 12, 4, 23, 5, 2),   Data(2, 11, 13, 4, 23, 5, 2),  Data(2, 11, 13, 4, 23, 5, 2),
      Data(3, 11, 13, 4, 24, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),    Data(2, 7, 13, 4, 23, 5, 2),   Data(0, 1, 13, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 11, 4, 21, 5, 2),    Data(0, 1, 11, 4, 21, 5, 2),   Data(0, 1, 12, 3, 21, 5, 2),
      Data(0, 1, 12, 4, 24, 5, 2),    Data(0, 7, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 1, 13, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 7, 13, 4, 23, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 6, 13, 4, 23, 5, 2),   Data(0, 1, 13, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 1, 14, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 7, 15, 4, 29, 5, 3),    Data(0, 7, 15, 4, 23, 5, 2),   Data(0, 1, 13, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 1, 14, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 7, 13, 4, 23, 5, 2),    Data(0, 0, 15, 4, 29, 5, 3),   Data(4, 13, 12, 4, 24, 5, 2),
      Data(2, 12, 12, 4, 23, 5, 2),   Data(3, 12, 14, 4, 23, 5, 2),   Data(2, 11, 12, 4, 24, 5, 2),  Data(2, 11, 12, 4, 23, 5, 2),
      Data(2, 11, 12, 4, 23, 5, 2),   Data(2, 11, 13, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(2, 7, 13, 4, 23, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),   Data(0, 7, 13, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 9, 3, 17, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),
      Data(0, 1, 17, 4, 28, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 7, 14, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 7, 13, 4, 23, 5, 2),    Data(0, 1, 15, 4, 26, 5, 2),    Data(0, 1, 15, 4, 25, 5, 2),   Data(0, 1, 14, 4, 24, 5, 2),
      Data(0, 1, 14, 4, 24, 5, 2),    Data(0, 1, 14, 4, 24, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),   Data(0, 8, 13, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 7, 13, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 1, 12, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 7, 12, 4, 24, 5, 2),   Data(0, 7, 13, 4, 24, 5, 2),
      Data(0, 7, 13, 4, 24, 5, 2),    Data(0, 7, 12, 4, 24, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),   Data(0, 7, 14, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 7, 13, 4, 23, 5, 2),    Data(0, 8, 14, 4, 23, 5, 2),   Data(0, 8, 13, 4, 23, 5, 2),
      Data(0, 14, 13, 4, 23, 5, 2),   Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 8, 13, 4, 23, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 8, 13, 4, 23, 5, 2),    Data(0, 7, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 7, 14, 4, 23, 5, 2),
      Data(0, 1, 12, 4, 25, 5, 2),    Data(0, 1, 15, 4, 29, 5, 2),    Data(0, 1, 14, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 7, 14, 4, 23, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),   Data(0, 1, 14, 4, 23, 5, 2),
      Data(0, 1, 11, 4, 21, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 16, 4, 29, 7, 3),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(1, 13, 9, 3, 17, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 14, 4, 25, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 7, 13, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 15, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 7, 13, 4, 23, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 1, 14, 4, 23, 5, 2),
      Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 7, 13, 4, 23, 5, 2),   Data(0, 7, 12, 4, 24, 5, 2),
      Data(0, 7, 13, 4, 23, 5, 2),    Data(0, 1, 14, 4, 28, 5, 3),    Data(0, 7, 13, 4, 24, 5, 2),   Data(0, 1, 14, 4, 23, 5, 2),
      Data(0, 7, 14, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 7, 13, 4, 23, 5, 2),
      Data(0, 1, 11, 4, 21, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 1, 13, 4, 25, 6, 2),   Data(0, 7, 13, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 1, 14, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 7, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 21, 4, 23, 5, 2),
      Data(0, 7, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 1, 13, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 1, 15, 4, 23, 5, 2),   Data(0, 7, 13, 4, 23, 5, 2),
      Data(0, 0, 15, 4, 29, 5, 3),    Data(0, 0, 13, 4, 25, 5, 2),    Data(0, 7, 13, 4, 24, 5, 2),   Data(0, 1, 15, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 1, 14, 4, 23, 5, 2),
      Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 7, 14, 4, 25, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 14, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(2, 15, 13, 4, 23, 5, 2),   Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 1, 13, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 14, 4, 23, 5, 4),   Data(0, 1, 13, 4, 23, 5, 2),
      Data(0, 7, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 14, 4, 23, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 15, 4, 23, 5, 2),    Data(0, 7, 13, 4, 23, 5, 2),   Data(0, 1, 17, 4, 23, 5, 2),
      Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),   Data(0, 7, 13, 4, 23, 5, 2),
      Data(0, 1, 13, 4, 25, 5, 2),    Data(0, 7, 13, 4, 23, 5, 2),    Data(0, 7, 12, 4, 24, 5, 2),   Data(0, 1, 13, 4, 23, 5, 2),
      Data(0, 8, 16, 4, 29, 5, 2),    Data(0, 7, 15, 4, 26, 5, 2),    Data(0, 7, 13, 4, 24, 5, 2),   Data(0, 1, 14, 4, 24, 5, 2),
      Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 7, 12, 4, 24, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),   Data(0, 1, 14, 4, 23, 5, 2),
      Data(0, 7, 13, 4, 23, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 9, 15, 4, 27, 5, 2),    Data(0, 0, 14, 4, 25, 5, 2),    Data(0, 0, 15, 4, 24, 5, 2),   Data(0, 0, 12, 4, 21, 5, 2),
      Data(1, 0, 12, 4, 21, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 9, 15, 4, 26, 5, 3),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 7, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 7, 14, 4, 23, 5, 2),    Data(0, 7, 13, 4, 23, 5, 2),   Data(0, 1, 16, 4, 24, 5, 2),
      Data(0, 7, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 25, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),   Data(0, 1, 13, 4, 23, 5, 2),
      Data(0, 8, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 5, 9, 3, 17, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 22, 4, 30, 5, 2),    Data(0, 7, 13, 4, 25, 5, 2),    Data(0, 1, 14, 4, 24, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 7, 13, 4, 24, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 1, 13, 4, 24, 5, 2),   Data(0, 1, 13, 4, 26, 5, 2),
      Data(0, 1, 13, 4, 24, 5, 2),    Data(0, 1, 13, 4, 26, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),   Data(0, 7, 13, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 26, 5, 2),    Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 12, 4, 24, 7, 5),   Data(0, 0, 14, 4, 23, 5, 2),
      Data(0, 0, 15, 4, 24, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),    Data(0, 0, 14, 4, 24, 5, 2),   Data(0, 0, 13, 3, 20, 5, 2),
      Data(4, 30, 13, 4, 25, 5, 2),   Data(0, 3, 14, 4, 24, 5, 2),    Data(0, 2, 13, 4, 24, 5, 2),   Data(0, 2, 13, 4, 26, 5, 2),
      Data(0, 2, 14, 4, 23, 5, 2),    Data(0, 2, 14, 4, 23, 5, 2),    Data(0, 2, 13, 4, 24, 5, 2),   Data(0, 15, 13, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 9, 3, 17, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),
      Data(0, 0, 20, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 15, 4, 23, 5, 2),   Data(0, 0, 14, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 14, 4, 23, 5, 2),
      Data(0, 14, 13, 4, 23, 5, 2),   Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 1, 13, 4, 23, 5, 2),
      Data(0, 11, 13, 4, 23, 5, 2),   Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 9, 14, 4, 23, 5, 2),
      Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 26, 5, 2),   Data(0, 8, 13, 4, 23, 5, 2),
      Data(0, 7, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 1, 13, 4, 24, 5, 2),
      Data(0, 1, 13, 4, 23, 5, 2),    Data(0, 1, 14, 4, 23, 5, 2),    Data(0, 1, 13, 4, 23, 5, 2),   Data(0, 11, 15, 4, 24, 5, 2),
      Data(0, 0, 15, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 3),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 18, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 0, 14, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 0, 14, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 24, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 19, 4, 23, 6, 2),   Data(0, 0, 13, 4, 24, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 14, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 1, 19, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 14, 4, 23, 5, 2),   Data(0, 0, 14, 4, 23, 5, 2),
      Data(0, 0, 20, 4, 23, 5, 3),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 14, 4, 23, 5, 2),
      Data(0, 0, 13, 4, 23, 5, 2),    Data(0, 0, 13, 4, 26, 5, 2),    Data(0, 0, 13, 4, 23, 5, 2),   Data(0, 0, 13, 4, 23, 5, 2),
    };
  }


  namespace DataGeneratorData
  {
    ReadOnlySpan<DataGeneratorDataRecord> DataSpan()
    {
      return ReadOnlySpanUtil::AsSpan(g_entries);
    }
  }

}
